@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-download.c,v 1.5 2004/03/17 09:02:31 marco Exp $
+ *  $Id: ephy-download.c,v 1.6 2004/07/03 22:52:28 xan Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -219,6 +219,13 @@ ephy_download_get_state (EphyDownload *download)
 	return klass->get_state (download);
 }
 
+char *
+ephy_download_get_mime (EphyDownload *download)
+{
+        EphyDownloadClass *klass = EPHY_DOWNLOAD_GET_CLASS (download);
+        return klass->get_mime (download);
+}
+
 void
 ephy_download_cancel (EphyDownload *download)
 {
