@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-encodings.c,v 1.10 2004/01/22 21:00:38 chpe Exp $
+ *  $Id: ephy-encodings.c,v 1.13 2004/05/01 20:46:59 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -115,10 +115,8 @@ encoding_entries [] =
 	{ N_("R_omanian (ISO-8859-16)"),            "ISO-8859-16",           LG_SE_EUROPEAN,	FALSE },
 	{ N_("South _European (ISO-8859-3)"),	    "ISO-8859-3",            LG_SE_EUROPEAN,	FALSE },
 	{ N_("Thai (TIS-_620)"),                    "TIS-620",               LG_THAI,		FALSE },
-#if MOZILLA_SNAPSHOT >= 10 
 	{ N_("Thai (IS_O-8859-11)"),                "iso-8859-11",           LG_THAI,		FALSE },
 	{ N_("_Thai (Windows-874)"),                "windows-874",           LG_THAI,		FALSE },
-#endif	
 	{ N_("Turkish (_IBM-857)"),                 "IBM857",                LG_TURKISH,	FALSE },
 	{ N_("Turkish (I_SO-8859-9)"),              "ISO-8859-9",            LG_TURKISH,	FALSE },
 	{ N_("Turkish (_MacTurkish)"),              "x-mac-turkish",         LG_TURKISH,	FALSE },
@@ -450,10 +448,6 @@ ephy_encodings_init (EphyEncodings *encodings)
 	encodings->priv->encodings = ephy_node_new_with_id (db, ENCODINGS_NODE_ID);
 	encodings->priv->detectors = ephy_node_new_with_id (db, DETECTORS_NODE_ID);
 
-	ephy_node_ref (encodings->priv->root);
-	ephy_node_ref (encodings->priv->encodings);
-	ephy_node_ref (encodings->priv->detectors);
-
 	/* now fill the db */
 	for (i = 0; i < n_encoding_entries; i++)
 	{
@@ -505,15 +499,6 @@ ephy_encoding_info_free (EphyEncodingInfo *info)
 	}
 }
 
-gboolean
-ephy_encoding_info_is_automatic (EphyEncodingInfo *info)
-{
-	g_return_val_if_fail (info != NULL, FALSE);
-
-	return (info->encoding_source < EMBED_ENCODING_PARENT_FORCED)
-		&& (info->forced_encoding == NULL || info->forced_encoding[0] == '\0');
-}
-
 EphyEncodings *
 ephy_encodings_new (void)
 {
