@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2003 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2004 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.cpp,v 1.22.4.13 2004/10/30 18:14:08 chpe Exp $
+ *  $Id: EphyBrowser.cpp,v 1.44 2004/06/25 13:38:36 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -23,81 +23,59 @@
 #endif
 
 #include "EphyBrowser.h"
-#include "GlobalHistory.h"
+#include "EphyUtils.h"
+#include "MozillaPrivate.h"
 #include "ephy-embed.h"
 #include "ephy-string.h"
 #include "ephy-debug.h"
+#include "print-dialog.h"
 
 #include <gtkmozembed_internal.h>
 #include <unistd.h>
 
-#include "nsICommandManager.h"
+#include "nsIInterfaceRequestorUtils.h"
+#include "nsIURI.h"
+#include "nsISimpleEnumerator.h"
+
 #include "nsIContentViewer.h"
-#include "nsIGlobalHistory.h"
-#include "nsIDocShellHistory.h"
 #include "nsIWebBrowserFind.h"
 #include "nsIWebBrowserFocus.h"
-#include "nsIDocument.h"
-#include "nsISHEntry.h"
-#include "nsISHistoryInternal.h"
-#include "nsIHistoryEntry.h"
+#include "nsICommandManager.h"
 #include "nsIWebBrowserPrint.h"
-#include "nsIURI.h"
-#include "nsIMarkupDocumentViewer.h"
-#include "nsIComponentManager.h"
-#include "nsIScriptGlobalObject.h"
-#include "nsIDOMWindowInternal.h"
-#include "nsIInterfaceRequestor.h"
-#include "nsIFocusController.h"
-#include "nsIWebBrowserPersist.h"
-#include "nsCWebBrowserPersist.h"
-#include "nsNetUtil.h"
-#include "nsIChromeEventHandler.h"
-#include "nsIDOMDocument.h"
-#include "nsIDOMDocumentStyle.h"
-#include "nsIDOMEvent.h"
-#include "nsIDOMNSEvent.h"
-#include "nsIDOMEventTarget.h"
-#include "nsIDOMNode.h"
-#include "nsIDOMElement.h"
-#include "nsIDOMPopupBlockedEvent.h"
+#include "nsIDocCharset.h"
 #include "nsIDocShellTreeItem.h"
 #include "nsIDocShellTreeNode.h"
 #include "nsIDocShellTreeOwner.h"
-#include "nsICSSLoader.h"
-#include "nsICSSStyleSheet.h"
-#include "nsICSSLoaderObserver.h"
-#include "nsIDocumentObserver.h"
-#include "nsCWebBrowser.h"
-#include "nsReadableUtils.h"
-#include "nsIDOMNSHTMLDocument.h"
+#include "nsIWebPageDescriptor.h"
+#include "nsISHEntry.h"
+#include "nsIHistoryEntry.h"
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDOMHTMLCollection.h"
 #include "nsIDOMHTMLElement.h"
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMHTMLInputElement.h"
 #include "nsIDOMHTMLTextAreaElement.h"
-#include "nsIDeviceContext.h"
-#if MOZILLA_SNAPSHOT >= 20
-#include "nsPresContext.h"
-#else
-#include "nsIPresContext.h"
-#endif
-#include "nsIAtom.h"
-#include "nsIDocumentCharsetInfo.h"
-#include "nsPromiseFlatString.h"
-#include "nsString.h"
-#include "ContentHandler.h"
-#include "MozillaPrivate.h"
-#include "print-dialog.h"
-#include "nsIScriptSecurityManager.h"
-#include "nsIServiceManager.h"
-#include "nsIContentPolicy.h"
-
-#if MOZILLA_SNAPSHOT >= 17
+#include "nsIDOMDocument.h"
+#include "nsIDOM3Document.h"
+#include "nsIDOMEvent.h"
+#include "nsIDOMEventTarget.h"
+#include "nsIDOMPopupBlockedEvent.h"
+#include "nsIDOMNode.h"
+#include "nsIDOMElement.h"
 #include "nsIDOMWindow2.h"
+#include "nsEmbedString.h"
+#include "nsMemory.h"
+
+#ifdef ALLOW_PRIVATE_API
+#include "nsIMarkupDocumentViewer.h"
 #endif
 
+static PRUnichar DOMLinkAdded[] = { 'D', 'O', 'M', 'L', 'i', 'n', 'k',
+				    'A', 'd', 'd', 'e', 'd', '\0' };
+static PRUnichar DOMPopupBlocked[] = { 'D', 'O', 'M', 'P', 'o', 'p',
+				       'u', 'p', 'B', 'l', 'o', 'c',
+				       'k', 'e', 'd', '\0' };
+
 EphyEventListener::EphyEventListener(void)
 : mOwner(nsnull)
 {
@@ -118,153 +96,115 @@ EphyEventListener::Init(EphyEmbed *aOwner)
 	return NS_OK;
 }
 
-NS_IMETHODIMP
-EphyFaviconEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
+nsresult
+EphyFaviconEventListener::HandleFaviconLink (nsIDOMNode *node)
 {
-	nsCOMPtr<nsIDOMEventTarget> eventTarget;
-	aDOMEvent->GetTarget(getter_AddRefs(eventTarget));
-
-	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(eventTarget);
-	NS_ENSURE_TRUE (node, NS_ERROR_FAILURE);
+	nsresult result;
 
 	nsCOMPtr<nsIDOMElement> linkElement;
 	linkElement = do_QueryInterface (node);
 	if (!linkElement) return NS_ERROR_FAILURE;
 
-	nsresult rv;
-	NS_NAMED_LITERAL_STRING(attr_rel, "rel");
-	nsAutoString value;
-	rv = linkElement->GetAttribute (attr_rel, value);
-	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+	PRUnichar relAttr[] = { 'r', 'e', 'l', '\0' };
+	nsEmbedString value;
+	result = linkElement->GetAttribute (nsEmbedString(relAttr), value);
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
 
-	if (value.EqualsIgnoreCase("SHORTCUT ICON") ||
-	    value.EqualsIgnoreCase("ICON"))
+	nsEmbedCString rel;
+	NS_UTF16ToCString (value, NS_CSTRING_ENCODING_UTF8, rel);	
+
+	if (g_ascii_strcasecmp (rel.get(), "SHORTCUT ICON") == 0 ||
+	    g_ascii_strcasecmp (rel.get(), "ICON") == 0)
 	{
-		NS_NAMED_LITERAL_STRING(attr_href, "href");
-		nsAutoString value;
-		rv = linkElement->GetAttribute (attr_href, value);
-		if (NS_FAILED (rv) || value.IsEmpty()) return NS_ERROR_FAILURE;
+		PRUnichar hrefAttr[] = { 'h', 'r', 'e', 'f', '\0' };
+		nsEmbedString value;
+		result = linkElement->GetAttribute (nsEmbedString (hrefAttr), value);
+		if (NS_FAILED (result) || !value.Length()) return NS_ERROR_FAILURE;
+
+		nsEmbedCString link;
+		NS_UTF16ToCString (value, NS_CSTRING_ENCODING_UTF8, link);
 
 		nsCOMPtr<nsIDOMDocument> domDoc;
 		node->GetOwnerDocument(getter_AddRefs(domDoc));
 		NS_ENSURE_TRUE (domDoc, NS_ERROR_FAILURE);
 
-		nsCOMPtr<nsIDocument> doc = do_QueryInterface (domDoc);
+		nsCOMPtr<nsIDOM3Document> doc = do_QueryInterface (domDoc);
 		NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT > 13
-		nsIURI *docUri;
-		docUri = doc->GetDocumentURI ();
-#elif MOZILLA_SNAPSHOT > 11
-		nsIURI *docUri;
-		docUri = doc->GetDocumentURL ();
-#else
-		nsCOMPtr<nsIURI> docUri;
-		doc->GetDocumentURL (getter_AddRefs (docUri));
-#endif
-		NS_ENSURE_TRUE (docUri, NS_ERROR_FAILURE);
+		nsEmbedString spec;
+		result = doc->GetDocumentURI (spec);
+		NS_ENSURE_SUCCESS (result, result);
 
-		const nsACString &link = NS_ConvertUCS2toUTF8(value);
-		nsCAutoString faviconUrl;
-		rv = docUri->Resolve (link, faviconUrl);
-		NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+		nsCOMPtr<nsIURI> uri;
+		result = EphyUtils::NewURI (getter_AddRefs(uri), spec);
+		NS_ENSURE_SUCCESS (result, result);
 
-		nsCOMPtr<nsIURI> favUri;
-		NS_NewURI (getter_AddRefs (favUri), faviconUrl);
-		NS_ENSURE_TRUE (favUri, NS_ERROR_FAILURE);
-
-		/* check if load is allowed */
-		nsCOMPtr<nsIScriptSecurityManager> secMan
-			(do_GetService("@mozilla.org/scriptsecuritymanager;1"));
-		/* refuse if we can't check */
-		NS_ENSURE_TRUE (secMan, NS_OK);
-
-		rv = secMan->CheckLoadURI(docUri, favUri,
-					  nsIScriptSecurityManager::STANDARD);
-		/* failure means it didn't pass the security check */
-		if (NS_FAILED (rv)) return NS_OK;
-
-		/* security check passed, now check with content policy */
-		nsCOMPtr<nsIContentPolicy> policy =
-			do_GetService("@mozilla.org/layout/content-policy;1");
-		/* refuse if we can't check */
-		NS_ENSURE_TRUE (policy, NS_OK);
-
-		PRBool shouldLoad = PR_FALSE;
-		rv = policy->ShouldLoad (nsIContentPolicy::IMAGE,
-					 favUri, eventTarget,
-					 nsnull /* FIXME: DOM window*/,
-					 &shouldLoad);
-		NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
-		if (!shouldLoad) return NS_OK;
-
-		/* ok, we accept this as a valid favicon for this site */
-		char *url = g_strdup (faviconUrl.get());
+		nsEmbedCString favicon_url;
+		result = uri->Resolve (link, favicon_url);
+		if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+		
+		char *url = g_strdup (favicon_url.get());
 		g_signal_emit_by_name (mOwner, "ge_favicon", url);
 		g_free (url);
 	}
 
 	return NS_OK;
-}
+}	
 
 NS_IMETHODIMP
-EphyPopupEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
+EphyFaviconEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
 {
-	nsCOMPtr<nsIDOMPopupBlockedEvent> popupEvent =
-		do_QueryInterface(aDOMEvent);
-	if (popupEvent)
-	{
-		g_signal_emit_by_name (mOwner, "ge_popup_blocked");
-	}
+	nsCOMPtr<nsIDOMEventTarget> eventTarget;
+
+	aDOMEvent->GetTarget(getter_AddRefs(eventTarget));
+
+	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(eventTarget);
+	NS_ENSURE_TRUE (node, NS_ERROR_FAILURE);
+
+	HandleFaviconLink (node);
 
 	return NS_OK;
 }
 
-#if MOZILLA_SNAPSHOT >= 18
 NS_IMETHODIMP
-EphyModalAlertEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
+EphyPopupBlockEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
 {
-	NS_ENSURE_TRUE (mOwner, NS_ERROR_FAILURE);
-
-	/* make sure the event is trusted */
-	nsCOMPtr<nsIDOMNSEvent> nsEvent (do_QueryInterface (aDOMEvent));
-	NS_ENSURE_TRUE (nsEvent, NS_ERROR_FAILURE);
-	PRBool isTrusted = PR_FALSE;
-	nsEvent->GetIsTrusted (&isTrusted);
-	if (!isTrusted) return NS_OK;
-
 	nsresult rv;
-	nsAutoString type;
-	rv = aDOMEvent->GetType (type);
-	NS_ENSURE_SUCCESS (rv, rv);
 
-	if (type.Equals(NS_LITERAL_STRING("DOMWillOpenModalDialog")))
-	{
-		gboolean retval = FALSE;
-		g_signal_emit_by_name (mOwner, "ge-modal-alert", &retval);
+	NS_ENSURE_TRUE (mOwner != NULL, NS_ERROR_FAILURE);
 
-		/* suppress alert */
-		if (retval)
-		{
-			aDOMEvent->PreventDefault ();
-			aDOMEvent->StopPropagation ();
-		}
-	}
-	else if (type.Equals(NS_LITERAL_STRING("DOMModalDialogClosed")))
-	{
-		g_signal_emit_by_name (mOwner, "ge-modal-alert-closed");
-	}
+	nsCOMPtr<nsIDOMPopupBlockedEvent> popupEvent =
+		do_QueryInterface (aDOMEvent, &rv);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIURI> popupWindowURI;
+	rv = popupEvent->GetPopupWindowURI (getter_AddRefs (popupWindowURI));
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+	NS_ENSURE_SUCCESS (popupWindowURI != NULL, NS_ERROR_FAILURE);
+
+	nsEmbedCString popupWindowURIString;
+	rv = popupWindowURI->GetSpec (popupWindowURIString);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	nsEmbedString popupWindowFeatures;
+	rv = popupEvent->GetPopupWindowFeatures (popupWindowFeatures);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	nsEmbedCString popupWindowFeaturesString;
+	NS_UTF16ToCString (popupWindowFeatures,
+			   NS_CSTRING_ENCODING_UTF8,
+			   popupWindowFeaturesString);
+
+	g_signal_emit_by_name(mOwner, "ge_popup_blocked",
+			      popupWindowURIString.get(),
+			      popupWindowFeaturesString.get());
 
 	return NS_OK;
 }
-#endif
 
 EphyBrowser::EphyBrowser ()
 : mFaviconEventListener(nsnull)
-, mPopupEventListener(nsnull)
-#if MOZILLA_SNAPSHOT >= 18
-, mModalAlertListener(nsnull)
-#endif
+, mPopupBlockEventListener(nsnull)
 , mInitialized(PR_FALSE)
 {
 }
@@ -294,24 +234,15 @@ nsresult EphyBrowser::Init (GtkMozEmbed *mozembed)
 	mFaviconEventListener = new EphyFaviconEventListener();
 	if (!mFaviconEventListener) return NS_ERROR_OUT_OF_MEMORY;
 
-	mPopupEventListener = new EphyPopupEventListener();
-	if (!mPopupEventListener) return NS_ERROR_OUT_OF_MEMORY;
-
 	rv = mFaviconEventListener->Init (EPHY_EMBED (mozembed));
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
-	rv = mPopupEventListener->Init (EPHY_EMBED (mozembed));
+	mPopupBlockEventListener = new EphyPopupBlockEventListener();
+	if (!mPopupBlockEventListener) return NS_ERROR_OUT_OF_MEMORY;
+
+	rv = mPopupBlockEventListener->Init (EPHY_EMBED (mozembed));
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT >= 18
-
-	mModalAlertListener = new EphyModalAlertEventListener ();
-	if (!mModalAlertListener) return NS_ERROR_OUT_OF_MEMORY;
-
-	rv = mModalAlertListener->Init (EPHY_EMBED (mozembed));
-	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
-#endif
-
  	rv = GetListener();
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
@@ -326,29 +257,15 @@ EphyBrowser::GetListener (void)
   	if (mEventReceiver) return NS_ERROR_FAILURE;
 
   	nsCOMPtr<nsIDOMWindow> domWindowExternal;
-  	mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindowExternal));
-
-#if MOZILLA_SNAPSHOT >= 17
-	nsCOMPtr<nsIDOMWindow2> domWindow (do_QueryInterface (domWindowExternal));
+  	mWebBrowser->GetContentDOMWindow (getter_AddRefs(domWindowExternal));
+  
+  	nsCOMPtr<nsIDOMWindow2> domWindow (do_QueryInterface (domWindowExternal));
 	NS_ENSURE_TRUE (domWindow, NS_ERROR_FAILURE);
-
-	nsCOMPtr<nsIDOMEventTarget> rootWindow;
-	domWindow->GetWindowRoot (getter_AddRefs (rootWindow));
-
-	mEventReceiver = do_QueryInterface (rootWindow);
-#else
-  	nsCOMPtr<nsIDOMWindowInternal> domWindow;
-        domWindow = do_QueryInterface(domWindowExternal);
 	
-	nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(domWindow));
-	NS_ENSURE_TRUE (piWin, NS_ERROR_FAILURE);
-
-  	nsCOMPtr<nsIChromeEventHandler> chromeHandler;
-  	piWin->GetChromeEventHandler(getter_AddRefs(chromeHandler));
-
-  	mEventReceiver = do_QueryInterface(chromeHandler);
-#endif /* MOZILLA_SNAPSHOT >= 17 */
+  	nsCOMPtr<nsIDOMEventTarget> rootWindow;
+  	domWindow->GetWindowRoot (getter_AddRefs(rootWindow));
 
+  	mEventReceiver = do_QueryInterface (rootWindow);
 	NS_ENSURE_TRUE (mEventReceiver, NS_ERROR_FAILURE);
 
 	return NS_OK;
@@ -364,22 +281,14 @@ EphyBrowser::AttachListeners(void)
 	nsCOMPtr<nsIDOMEventTarget> target;
 	target = do_QueryInterface (mEventReceiver);
 
-	rv = target->AddEventListener(NS_LITERAL_STRING("DOMLinkAdded"),
+	rv = target->AddEventListener(nsEmbedString(DOMLinkAdded),
 				      mFaviconEventListener, PR_FALSE);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
-	rv = target->AddEventListener(NS_LITERAL_STRING("DOMPopupBlocked"),
-				      mPopupEventListener, PR_FALSE);
+	rv = target->AddEventListener(nsEmbedString(DOMPopupBlocked),
+				      mPopupBlockEventListener, PR_FALSE);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT >= 18
-	rv = target->AddEventListener(NS_LITERAL_STRING("DOMWillOpenModalDialog"),
-				      mModalAlertListener, PR_TRUE);
-	rv |= target->AddEventListener(NS_LITERAL_STRING("DOMWillOpenModalDialog"),
-				       mModalAlertListener, PR_TRUE);
-	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
-#endif
-
 	return NS_OK;
 }
 
@@ -394,21 +303,12 @@ EphyBrowser::DetachListeners(void)
 	target = do_QueryInterface (mEventReceiver);
 	NS_ENSURE_TRUE (target, NS_ERROR_FAILURE);
 
-	rv = target->RemoveEventListener(NS_LITERAL_STRING("DOMLinkAdded"),
+	rv = target->RemoveEventListener(nsEmbedString(DOMLinkAdded),
 					 mFaviconEventListener, PR_FALSE);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
-	rv = target->RemoveEventListener(NS_LITERAL_STRING("DOMPopupBlocked"),
-					 mPopupEventListener, PR_FALSE);
-	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
-
-#if MOZILLA_SNAPSHOT >= 18
-	rv = target->RemoveEventListener(NS_LITERAL_STRING("DOMWillOpenModalDialog"),
-					 mModalAlertListener, PR_TRUE);
-	rv |= target->RemoveEventListener(NS_LITERAL_STRING("DOMWillOpenModalDialog"),
-					  mModalAlertListener, PR_TRUE);
-	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
-#endif
+	rv = target->RemoveEventListener(nsEmbedString(DOMPopupBlocked),
+					 mPopupBlockEventListener, PR_FALSE);
 
 	return NS_OK;
 }
@@ -560,73 +460,19 @@ nsresult EphyBrowser::Reload (PRUint32 flags)
 
 	return webNavigation->Reload (reloadFlags);
 }
- 
-nsresult EphyBrowser::SetZoom (float aZoom, PRBool reflow)
+
+nsresult EphyBrowser::SetZoom (float aZoom)
 {
-	if (!mWebBrowser) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
-	if (reflow)
-	{
-		nsCOMPtr<nsIContentViewer> contentViewer;	
-		GetContentViewer (getter_AddRefs(contentViewer));
-		NS_ENSURE_TRUE (contentViewer, NS_ERROR_FAILURE);
+	nsCOMPtr<nsIContentViewer> contentViewer;	
+	GetContentViewer (getter_AddRefs(contentViewer));
+	NS_ENSURE_TRUE (contentViewer, NS_ERROR_FAILURE);
 
-		nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
-		NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
+	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
+	NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
 
-		return mdv->SetTextZoom (aZoom);
-	}
-	else
-	{
-		nsCOMPtr<nsIDocShell> DocShell;
-		DocShell = do_GetInterface (mWebBrowser);
-		NS_ENSURE_TRUE (DocShell, NS_ERROR_FAILURE);
-
-		SetZoomOnDocshell (aZoom, DocShell);
-
-		nsCOMPtr<nsIDocShellTreeNode> docShellNode(do_QueryInterface(DocShell));
-		if (docShellNode)
-		{
-			PRInt32 i;
-			PRInt32 n;
-			docShellNode->GetChildCount(&n);
-			for (i=0; i < n; i++) 
-			{
-				nsCOMPtr<nsIDocShellTreeItem> child;
-				docShellNode->GetChildAt(i, getter_AddRefs(child));
-				nsCOMPtr<nsIDocShell> childAsShell(do_QueryInterface(child));
-				if (childAsShell) 
-				{
-					return SetZoomOnDocshell (aZoom, childAsShell);
-				}
-			}
-		}
-	}
-
-	return NS_OK;
-}
-
-nsresult EphyBrowser::SetZoomOnDocshell (float aZoom, nsIDocShell *DocShell)
-{
-#if MOZILLA_SNAPSHOT >= 20
-	nsCOMPtr<nsPresContext> PresContext;
-#else
-	nsCOMPtr<nsIPresContext> PresContext;
-#endif
-	DocShell->GetPresContext (getter_AddRefs(PresContext));
-	NS_ENSURE_TRUE (PresContext, NS_ERROR_FAILURE);
-
-#if MOZILLA_SNAPSHOT > 13
-	nsIDeviceContext *DeviceContext;
-	DeviceContext = PresContext->DeviceContext();
-	NS_ENSURE_TRUE (DeviceContext, NS_ERROR_FAILURE);
-#else				
-	nsCOMPtr<nsIDeviceContext> DeviceContext;
-	PresContext->GetDeviceContext (getter_AddRefs(DeviceContext));
-	NS_ENSURE_TRUE (DeviceContext, NS_ERROR_FAILURE);
-#endif
-
-	return DeviceContext->SetTextZoom (aZoom);
+	return mdv->SetTextZoom (aZoom);
 }
 
 nsresult EphyBrowser::GetContentViewer (nsIContentViewer **aViewer)
@@ -743,7 +589,7 @@ nsresult EphyBrowser::GetSHUrlAtIndex (PRInt32 index, nsACString &url)
 
 	nsresult result;
 	result = uri->GetSpec(url);
-	NS_ENSURE_TRUE (NS_SUCCEEDED (result) && !url.IsEmpty(), NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (NS_SUCCEEDED (result) && url.Length(), NS_ERROR_FAILURE);
 
 	return NS_OK;
 }
@@ -792,56 +638,57 @@ nsresult EphyBrowser::GetPageDescriptor(nsISupports **aPageDescriptor)
 	return NS_OK;
 }
 
+nsresult EphyBrowser::GetDOMWindow (nsIDOMWindow **aDOMWindow)
+{
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
+
+	NS_IF_ADDREF (*aDOMWindow = mDOMWindow);
+
+	return NS_OK;
+}
+
 nsresult EphyBrowser::GetDocumentUrl (nsACString &url)
 {
+	nsresult rv;
+
 	NS_ENSURE_TRUE (mDOMWindow, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIDOMDocument> DOMDocument;
 	mDOMWindow->GetDocument (getter_AddRefs(DOMDocument));
 	NS_ENSURE_TRUE (DOMDocument, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
+	nsCOMPtr<nsIDOM3Document> doc = do_QueryInterface(DOMDocument);
 	NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT > 13
-	nsIURI *uri;
-	uri = doc->GetDocumentURI ();
-#elif MOZILLA_SNAPSHOT > 11
-	nsIURI *uri;
-	uri = doc->GetDocumentURL ();
-#else
-	nsCOMPtr<nsIURI> uri;
-	doc->GetDocumentURL(getter_AddRefs(uri));
-#endif
-	NS_ENSURE_TRUE (uri, NS_ERROR_FAILURE);
+	nsEmbedString docURI;
+	rv = doc->GetDocumentURI (docURI);
+	NS_ENSURE_SUCCESS (rv, rv);
 
-	return uri->GetSpec (url);
+	NS_UTF16ToCString (docURI, NS_CSTRING_ENCODING_UTF8, url);
+
+	return NS_OK;
 }
 
 nsresult EphyBrowser::GetTargetDocumentUrl (nsACString &url)
 {
+	nsresult rv;
+
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
         nsCOMPtr<nsIDOMDocument> DOMDocument;
 	GetTargetDocument (getter_AddRefs(DOMDocument));
 	NS_ENSURE_TRUE (DOMDocument, NS_ERROR_FAILURE);
 
-        nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
+	nsCOMPtr<nsIDOM3Document> doc = do_QueryInterface(DOMDocument);
 	NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT > 13
-	nsIURI *uri;
-	uri = doc->GetDocumentURI ();
-#elif MOZILLA_SNAPSHOT > 11
-	nsIURI *uri;
-	uri = doc->GetDocumentURL ();
-#else
-        nsCOMPtr<nsIURI> uri;
-        doc->GetDocumentURL(getter_AddRefs(uri));
-#endif
-	NS_ENSURE_TRUE (uri, NS_ERROR_FAILURE);
+	nsEmbedString docURI;
+	rv = doc->GetDocumentURI (docURI);
+	NS_ENSURE_SUCCESS (rv, rv);
 
-	return uri->GetSpec (url);
+	NS_UTF16ToCString (docURI, NS_CSTRING_ENCODING_UTF8, url);
+
+	return NS_OK;
 }
 
 nsresult EphyBrowser::ForceEncoding (const char *encoding) 
@@ -855,14 +702,39 @@ nsresult EphyBrowser::ForceEncoding (const char *encoding)
 	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
 	NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
 
-	nsresult result;
-#if MOZILLA_SNAPSHOT > 9 
-	result = mdv->SetForceCharacterSet (nsDependentCString(encoding));
-#else
-	result = mdv->SetForceCharacterSet (NS_ConvertUTF8toUCS2(encoding).get());
-#endif
+	return mdv->SetForceCharacterSet (nsEmbedCString(encoding));
+}
 
-	return result;
+nsresult EphyBrowser::GetEncoding (nsACString &encoding)
+{
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIDocCharset> docCharset = do_GetInterface (mWebBrowser);
+	NS_ENSURE_TRUE (docCharset, NS_ERROR_FAILURE);
+
+	char *charset;
+	docCharset->GetCharset (&charset);
+	encoding = charset;
+	nsMemory::Free (charset);
+
+	return NS_OK;
+}
+
+nsresult EphyBrowser::GetForcedEncoding (nsACString &encoding)
+{
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIContentViewer> contentViewer;	
+	GetContentViewer (getter_AddRefs(contentViewer));
+	NS_ENSURE_TRUE (contentViewer, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
+	NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
+
+	nsresult result = mdv->GetForceCharacterSet (encoding);
+	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
+
+	return NS_OK;
 }
 
 nsresult EphyBrowser::PushTargetDocument (nsIDOMDocument *domDoc)
@@ -879,132 +751,6 @@ nsresult EphyBrowser::PopTargetDocument ()
 	return NS_OK;
 }
 
-nsresult EphyBrowser::GetEncodingInfo (EphyEncodingInfo **infoptr)
-{
-	nsresult result;
-	EphyEncodingInfo *info;
-
-	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
-
-	nsCOMPtr<nsIDOMDocument> domDoc;
-	GetTargetDocument (getter_AddRefs(domDoc));
-	NS_ENSURE_TRUE (domDoc, NS_ERROR_FAILURE);
-
-	nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc, &result);
-	NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
-
-	info = g_new0 (EphyEncodingInfo, 1);
-	*infoptr = info;
-
-	PRInt32 source;
-#if MOZILLA_SNAPSHOT > 11
-	source = doc->GetDocumentCharacterSetSource ();
-#else
-	result = doc->GetDocumentCharacterSetSource (&source);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-#endif
-	info->encoding_source = (EphyEncodingSource) source;
-
-	nsCOMPtr<nsIDocShell> ds;
-	ds = do_GetInterface (mWebBrowser);
-	NS_ENSURE_TRUE (ds, NS_ERROR_FAILURE);
-
-	nsCOMPtr<nsIDocumentCharsetInfo> ci;
-	result = ds->GetDocumentCharsetInfo (getter_AddRefs (ci));
-	NS_ENSURE_TRUE (ci, NS_ERROR_FAILURE);
-
-	nsCOMPtr<nsIAtom> atom;
-	ci->GetForcedCharset (getter_AddRefs (atom));
-	if (atom)
-	{
-		nsCAutoString atomstr;
-		atom->ToUTF8String (atomstr);
-		info->forced_encoding = g_strdup (atomstr.get());
-	}
-
-	ci->GetParentCharset (getter_AddRefs (atom));
-	if (atom)
-	{
-		nsCAutoString atomstr;
-		atom->ToUTF8String (atomstr);
-		info->parent_encoding = g_strdup (atomstr.get());
-	}
-
-	result = ci->GetParentCharsetSource (&source);
-	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
-	info->parent_encoding_source = (EphyEncodingSource) source;
-
-	nsCOMPtr<nsIContentViewer> contentViewer;	
-	ds->GetContentViewer (getter_AddRefs(contentViewer));
-	NS_ENSURE_TRUE (contentViewer, NS_ERROR_FAILURE);
-
-	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
-	NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
-
-#if MOZILLA_SNAPSHOT > 11
-	const nsACString& charsetEnc = doc->GetDocumentCharacterSet ();
-	NS_ENSURE_TRUE (!charsetEnc.IsEmpty(), NS_ERROR_FAILURE);
-
-	info->encoding = g_strdup (PromiseFlatCString(charsetEnc).get());
-#elif MOZILLA_SNAPSHOT >= 10
-	nsCAutoString charsetEnc;	
-	result = doc->GetDocumentCharacterSet (charsetEnc);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-
-	info->encoding = g_strdup (charsetEnc.get());
-#else
-	nsAutoString charsetEnc;
-	result = doc->GetDocumentCharacterSet (charsetEnc);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-
-	info->encoding = g_strdup (NS_ConvertUCS2toUTF8(charsetEnc).get());
-#endif
-
-#if MOZILLA_SNAPSHOT >= 10
-	nsCAutoString enc;
-	
-	result = mdv->GetDefaultCharacterSet (enc);
-	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
-	info->default_encoding = g_strdup (enc.get());
-
-	result = mdv->GetForceCharacterSet (enc);
-	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
-	info->forced_encoding = g_strdup (enc.get());
-
-	result = mdv->GetHintCharacterSet (enc);
-	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
-	info->hint_encoding = g_strdup (enc.get());
-
-	result = mdv->GetPrevDocCharacterSet (enc);
-	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
-	info->prev_doc_encoding = g_strdup (enc.get());
-#else
-	PRUnichar *str;
-
-	result = mdv->GetDefaultCharacterSet (&str);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-	info->default_encoding = g_strdup (NS_ConvertUCS2toUTF8(str).get());
-
-	result = mdv->GetForceCharacterSet (&str);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-	info->forced_encoding = g_strdup (NS_ConvertUCS2toUTF8(str).get());
-
-	result = mdv->GetHintCharacterSet (&str);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-	info->hint_encoding = g_strdup (NS_ConvertUCS2toUTF8(str).get());
-
-	result = mdv->GetPrevDocCharacterSet (&str);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-	info->prev_doc_encoding = g_strdup (NS_ConvertUCS2toUTF8(str).get());
-#endif
-
-	mdv->GetHintCharacterSetSource (&source);
-	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
-	info->hint_encoding_source = (EphyEncodingSource) source;
-
-	return NS_OK;
-}
-
 nsresult EphyBrowser::DoCommand (const char *command)
 {
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
@@ -1029,6 +775,29 @@ nsresult EphyBrowser::GetCommandState (const char *command, PRBool *enabled)
 
 #define NUM_MODIFIED_TEXTFIELDS_REQUIRED	2
 
+PRBool
+EphyBrowser::CompareFormsText (nsAString &aDefaultText, nsAString &aUserText)
+{
+	if (aDefaultText.Length() != aUserText.Length())
+	{
+		return FALSE;
+	}
+
+	/* Mozilla Bug 218277, 195946 and others */
+	const PRUnichar *text = aDefaultText.BeginReading();
+	for (PRUint32 i = 0; i < aDefaultText.Length(); i++)
+	{
+		if (text[i] == 0xa0)
+		{
+			aDefaultText.Replace (i, 1, ' ');
+		}
+	}
+
+	return (memcmp (aDefaultText.BeginReading(),
+		        aUserText.BeginReading(),
+		        aUserText.Length() * sizeof (PRUnichar)) == 0);
+}
+
 nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUint32 *aNumTextFields, PRBool *aHasTextArea)
 {
 	nsCOMPtr<nsIDOMHTMLDocument> htmlDoc = do_QueryInterface(aDomDoc);
@@ -1069,14 +838,11 @@ nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUi
 			nsCOMPtr<nsIDOMHTMLTextAreaElement> areaElement = do_QueryInterface (domNode);
 			if (areaElement)
 			{
-				nsAutoString default_text, user_text;
-				areaElement->GetDefaultValue (default_text);
-				areaElement->GetValue (user_text);
+				nsEmbedString defaultText, userText;
+				areaElement->GetDefaultValue (defaultText);
+				areaElement->GetValue (userText);
 
-				/* Mozilla Bug 218277, 195946 and others */
-				default_text.ReplaceChar(0xa0, ' ');
-
-				if (!user_text.Equals (default_text))
+				if (!CompareFormsText (defaultText, userText))
 				{
 					*aHasTextArea = PR_TRUE;
 					return NS_OK;
@@ -1088,35 +854,30 @@ nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUi
 			nsCOMPtr<nsIDOMHTMLInputElement> inputElement = do_QueryInterface(domNode);
 			if (!inputElement) continue;
 	
-			nsAutoString type;
+			nsEmbedString type;
 			inputElement->GetType(type);
 
-#if MOZILLA_SNAPSHOT >= 20
-			if (type.LowerCaseEqualsASCII("text"))
-#else
-			if (type.EqualsIgnoreCase("text"))
-#endif
+			nsEmbedCString cType;
+			NS_UTF16ToCString (type, NS_CSTRING_ENCODING_UTF8, cType);
+
+			if (g_ascii_strcasecmp (cType.get(), "text") == 0)
 			{
-				nsAutoString default_text, user_text;
+				nsEmbedString defaultText, userText;
 				PRInt32 max_length;
-				inputElement->GetDefaultValue (default_text);
-				inputElement->GetValue (user_text);
+				inputElement->GetDefaultValue (defaultText);
+				inputElement->GetValue (userText);
 				inputElement->GetMaxLength (&max_length);
 
-				/* Guard against arguably broken forms where
-				 * default_text is longer than maxlength
-				 * (user_text is cropped, default_text is not)
-				 * Mozilla bug 232057
+				/* There are forms for which defaultValue is longer than
+				 * userValue. Mozilla consider this not a bug [see WONTFIXed
+				 * bug 232057], but we need to check for this here.
 				 */
-				if (default_text.Length() > (PRUint32)max_length)
+				if (defaultText.Length() > (PRUint32)max_length)
 				{
-					default_text.Truncate (max_length);
+					defaultText.Cut (max_length, PR_UINT32_MAX);
 				}
 
-				/* Mozilla Bug 218277, 195946 and others */
-				default_text.ReplaceChar(0xa0, ' ');
-
-				if (!user_text.Equals (default_text))
+				if (!CompareFormsText (defaultText, userText))
 				{
 					(*aNumTextFields)++;
 					if (*aNumTextFields >= NUM_MODIFIED_TEXTFIELDS_REQUIRED)
