@@ -34,28 +34,30 @@
  * the provisions above, a recipient may use your version of this file under
  * the terms of any one of the MPL, the GPL or the LGPL.
  *
- * ***** END LICENSE BLOCK ***** */
+ * ***** END LICENSE BLOCK *****
+ *
+ * $Id: EphyHeaderSniffer.h,v 1.12 2004/06/03 18:51:32 marco Exp $
+ */
 
 #include "mozilla-embed-persist.h"
-#include "ephy-embed-single.h"
 
-#include "nsString.h"
-#include "nsIWebProgressListener.h"
-#include "nsIWebBrowserPersist.h"
-#include "nsIURI.h"
-#include "nsILocalFile.h"
-#include "nsIInputStream.h"
-#include "nsIDOMDocument.h"
-#include "nsIAuthPrompt.h"
-#include "nsIPromptService.h"
+#include <nsIWebProgressListener.h>
+#include <nsIWebBrowserPersist.h>
+#include <nsIURI.h>
+#include <nsILocalFile.h>
+#include <nsIInputStream.h>
+#include <nsIDOMDocument.h>
+#include <nsIAuthPrompt.h>
+#include <nsIPromptService.h>
+#include <nsEmbedString.h>
 
 class EphyHeaderSniffer : public nsIWebProgressListener,
 			  public nsIAuthPrompt
 {
 public:
 	EphyHeaderSniffer (nsIWebBrowserPersist* aPersist, MozillaEmbedPersist *aEmbedPersist,
-		           nsIFile* aFile, nsIURI* aURL, nsIDOMDocument* aDocument,
-			   nsIInputStream* aPostData, EphyEmbedSingle *single);
+		           nsIFile* aFile, nsIURI* aURL,
+                           nsIDOMDocument* aDocument, nsIInputStream* aPostData);
 	virtual ~EphyHeaderSniffer ();
 
 	NS_DECL_ISUPPORTS
@@ -71,14 +73,13 @@ protected:
 private:
 	nsIWebBrowserPersist*      mPersist; /* Weak. It owns us as a listener. */
 	MozillaEmbedPersist       *mEmbedPersist;
-	EphyEmbedSingle		  *mSingle;
 	nsCOMPtr<nsIFile>          mTmpFile;
 	nsCOMPtr<nsIURI>           mURL;
 	nsCOMPtr<nsIURI>           mOriginalURI;
 	nsCOMPtr<nsIDOMDocument>   mDocument;
 	nsCOMPtr<nsIInputStream>   mPostData;
-	nsCString                  mContentType;
-	nsCString                  mContentDisposition;
+	nsEmbedCString             mContentType;
+	nsEmbedCString             mContentDisposition;
 	nsCOMPtr<nsIAuthPrompt>    mAuthPrompt;
 };
 
