@@ -16,9 +16,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: print-dialog.c,v 1.23.4.4 2004/07/17 14:36:02 chpe Exp $
+ *  $Id: print-dialog.c,v 1.29 2004/06/25 13:38:36 marco Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "print-dialog.h"
 #include "ephy-embed-dialog.h"
 #include "ephy-file-chooser.h"
@@ -131,6 +135,8 @@ void ephy_print_dialog_response_cb		(GtkWidget *widget,
 						 EphyDialog *dialog);
 void ephy_print_dialog_browse_button_cb		(GtkWidget *widget,
 						 EphyDialog *dialog);
+void ephy_print_setup_dialog_close_button_cb	(GtkWidget *widget,
+						 EphyDialog *dialog);
 void ephy_print_setup_dialog_help_button_cb	(GtkWidget *widget,
 						 EphyDialog *dialog);
 
@@ -162,7 +168,11 @@ ephy_print_get_print_info (void)
 	filename = eel_gconf_get_string (print_props[FILE_PROP].pref);
 	if (filename != NULL)
 	{
-		info->file = gnome_vfs_expand_initial_tilde (filename);
+		char *expanded;
+
+		expanded = gnome_vfs_expand_initial_tilde (filename);
+		info->file = g_filename_to_utf8 (expanded, -1, NULL, NULL, NULL);
+		g_free (expanded);
 	}
 	else
 	{
@@ -251,27 +261,41 @@ ephy_print_dialog_browse_button_cb (GtkWidget *widget,
 {
 	GtkWidget *parent;
 	EphyFileChooser *fc;
+	GtkFileFilter *filter;
 
 	parent = ephy_dialog_get_control (dialog, print_props[WINDOW_PROP].id);
 
 	fc = ephy_file_chooser_new (_("Print to"),
 				    GTK_WIDGET (parent),
 				    GTK_FILE_CHOOSER_ACTION_SAVE,
-				    CONF_PRINT_DIR);
+				    CONF_PRINT_DIR, EPHY_FILE_FILTER_NONE);
+
+	filter = ephy_file_chooser_add_mime_filter (fc, _("Postscript files"),
+					   "application/postscript", NULL);
+
+	ephy_file_chooser_add_pattern_filter (fc, _("All files"), "*", NULL);
+
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (fc), filter);
 
 	g_signal_connect (GTK_DIALOG (fc), "response",
 			  G_CALLBACK (print_filechooser_response_cb),
 			  dialog);
 
-	gtk_window_set_modal (GTK_WINDOW (fc), TRUE);
 	gtk_widget_show (GTK_WIDGET (fc));
 }
 
 void
-ephy_print_setup_dialog_help_button_cb (GtkWidget *widget,
-					EphyDialog *dialog)
+ephy_print_setup_dialog_close_button_cb (GtkWidget *widget,
+					 EphyDialog *dialog)
 {
-	ephy_gui_help (GTK_WINDOW (widget), "epiphany", "using-print-setup");
+	g_object_unref (dialog);
+}
+
+void
+ephy_print_setup_dialog_help_button_cb (GtkWidget *widget,
+					 EphyDialog *dialog)
+{
+	ephy_gui_help (GTK_WINDOW (dialog), "epiphany", "using-print-setup");
 }
 
 EphyDialog *
