@@ -35,6 +35,9 @@ enum
   TOOLBAR_ADDED,
   TOOLBAR_CHANGED,
   TOOLBAR_REMOVED,
+  GET_ITEM_TYPE,
+  GET_ITEM_ID,
+  GET_ITEM_DATA,
   LAST_SIGNAL
 };
 
@@ -123,13 +126,13 @@ egg_toolbars_model_to_xml (EggToolbarsModel *t)
 	    }
 	  else
 	    {
-	      char *name;
+	      char *data;
 
 	      node = xmlNewChild (tnode, NULL, "toolitem", NULL);
-	      name = egg_toolbars_model_get_item_name (t, item->type, item->id);
+	      data = egg_toolbars_model_get_item_data (t, item->type, item->id);
 	      xmlSetProp (node, "type", item->type);
-	      xmlSetProp (node, "name", name);
-	      g_free (name);
+	      xmlSetProp (node, "name", data);
+	      g_free (data);
 	    }
 	}
     }
@@ -267,7 +270,7 @@ egg_toolbars_model_get_flags (EggToolbarsModel *t,
   EggToolbarsToolbar *toolbar;
 
   toolbar_node = g_node_nth_child (t->priv->toolbars, toolbar_position);
-  g_return_val_if_fail (toolbar_node != NULL, -1);
+  g_return_val_if_fail (toolbar_node != NULL, 0);
 
   toolbar = toolbar_node->data;
 
@@ -306,7 +309,7 @@ egg_toolbars_model_add_separator (EggToolbarsModel *t,
   g_return_if_fail (EGG_IS_TOOLBARS_MODEL (t));
 
   parent_node = g_node_nth_child (t->priv->toolbars, toolbar_position);
-  item = toolbars_item_new ("separator", "separator", TRUE);
+  item = toolbars_item_new ("separator", EGG_TOOLBAR_ITEM_TYPE, TRUE);
   node = g_node_new (item);
   g_node_insert (parent_node, position, node);
 
@@ -371,10 +374,10 @@ parse_item_list (EggToolbarsModel *t,
 	        {
 	          egg_toolbars_model_add_item (t, position, -1, id, type);
                 }
-	      g_free (id);
-	    }
+              g_free (id);
+            }
 	  xmlFree (name);
-	  xmlFree (type);
+          xmlFree (type);
 	}
       else if (xmlStrEqual (child->name, "separator"))
 	{
@@ -423,7 +426,7 @@ parse_toolbars (EggToolbarsModel *t,
 	  xmlFree (name);
 
 	  style = xmlGetProp (child, "style");
-	  if (style && strcmp (style, "icons-only") == 0)
+	  if (style && xmlStrEqual (style, "icons-only"))
 	    {
 	      egg_toolbars_model_set_flags (t, EGG_TB_MODEL_ICONS_ONLY, 0);
 	    }
@@ -445,6 +448,8 @@ egg_toolbars_model_load (EggToolbarsModel *t,
 
   g_return_val_if_fail (EGG_IS_TOOLBARS_MODEL (t), FALSE);
 
+  if (!xml_file || !g_file_test (xml_file, G_FILE_TEST_EXISTS)) return FALSE;
+
   doc = xmlParseFile (xml_file);
   if (doc == NULL)
   {
@@ -453,7 +458,6 @@ egg_toolbars_model_load (EggToolbarsModel *t,
   }
   root = xmlDocGetRootElement (doc);
 
-  t->priv->toolbars = g_node_new (NULL);
   parse_toolbars (t, root->children);
 
   xmlFreeDoc (doc);
@@ -464,18 +468,18 @@ egg_toolbars_model_load (EggToolbarsModel *t,
 static char *
 impl_get_item_id (EggToolbarsModel *t,
 		  const char       *type,
-		  const char       *name)
+		  const char       *data)
 {
   if (strcmp (type, EGG_TOOLBAR_ITEM_TYPE) == 0)
     {
-      return g_strdup (name);
+      return g_strdup (data);
     }
 
   return NULL;
 }
 
 static char *
-impl_get_item_name (EggToolbarsModel *t,
+impl_get_item_data (EggToolbarsModel *t,
 		    const char       *type,
 		    const char       *id)
 {
@@ -499,6 +503,23 @@ impl_get_item_type (EggToolbarsModel *t,
   return NULL;
 }
 
+static gboolean
+_egg_accumulator_STRING (GSignalInvocationHint *ihint,
+                         GValue                *return_accu,
+                         const GValue          *handler_return,
+                         gpointer               dummy)
+{
+  gboolean continue_emission;
+  const char *retval;
+
+  retval = g_value_get_string (handler_return);
+  g_value_set_string (return_accu, retval);
+  continue_emission = !retval || !retval[0];
+  
+  return continue_emission;
+}
+
+
 static void
 egg_toolbars_model_class_init (EggToolbarsModelClass *klass)
 {
@@ -510,7 +531,7 @@ egg_toolbars_model_class_init (EggToolbarsModelClass *klass)
 
   klass->add_item = impl_add_item;
   klass->get_item_id = impl_get_item_id;
-  klass->get_item_name = impl_get_item_name;
+  klass->get_item_data = impl_get_item_data;
   klass->get_item_type = impl_get_item_type;
 
   egg_toolbars_model_signals[ITEM_ADDED] =
@@ -548,6 +569,30 @@ egg_toolbars_model_class_init (EggToolbarsModelClass *klass)
 		  G_STRUCT_OFFSET (EggToolbarsModelClass, toolbar_changed),
 		  NULL, NULL, g_cclosure_marshal_VOID__INT,
 		  G_TYPE_NONE, 1, G_TYPE_INT);
+  egg_toolbars_model_signals[GET_ITEM_TYPE] =
+    g_signal_new ("get_item_type",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (EggToolbarsModelClass, get_item_type),
+		  _egg_accumulator_STRING, NULL,
+		  _egg_marshal_STRING__POINTER,
+		  G_TYPE_STRING, 1, G_TYPE_POINTER);
+  egg_toolbars_model_signals[GET_ITEM_ID] =
+    g_signal_new ("get_item_id",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (EggToolbarsModelClass, get_item_id),
+		  _egg_accumulator_STRING, NULL,
+		  _egg_marshal_STRING__STRING_STRING,
+		  G_TYPE_STRING, 2, G_TYPE_STRING, G_TYPE_STRING);
+  egg_toolbars_model_signals[GET_ITEM_DATA] =
+    g_signal_new ("get_item_data",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (EggToolbarsModelClass, get_item_data),
+		  _egg_accumulator_STRING, NULL,
+		  _egg_marshal_STRING__STRING_STRING,
+		  G_TYPE_STRING, 2, G_TYPE_STRING, G_TYPE_STRING);
 
   g_type_class_add_private (object_class, sizeof (EggToolbarsModelPrivate));
 }
@@ -557,7 +602,7 @@ egg_toolbars_model_init (EggToolbarsModel *t)
 {
   t->priv =EGG_TOOLBARS_MODEL_GET_PRIVATE (t);
 
-  t->priv->toolbars = NULL;
+  t->priv->toolbars = g_node_new (NULL);
 }
 
 static void
@@ -635,27 +680,37 @@ egg_toolbars_model_n_items (EggToolbarsModel *t,
   return g_node_n_children (toolbar);
 }
 
-const char *
+void
 egg_toolbars_model_item_nth (EggToolbarsModel *t,
 			     int	       toolbar_position,
 			     int               position,
-			     gboolean         *is_separator)
+			     gboolean         *is_separator,
+			     const char      **id,
+			     const char      **type)
 {
   GNode *toolbar;
   GNode *item;
   EggToolbarsItem *idata;
 
   toolbar = g_node_nth_child (t->priv->toolbars, toolbar_position);
-  g_return_val_if_fail (toolbar != NULL, NULL);
+  g_return_if_fail (toolbar != NULL);
 
   item = g_node_nth_child (toolbar, position);
-  g_return_val_if_fail (item != NULL, NULL);
+  g_return_if_fail (item != NULL);
 
   idata = item->data;
 
   *is_separator = idata->separator;
 
-  return idata->id;
+  if (id)
+    {
+      *id = idata->id;
+    }
+
+  if (type)
+    {
+      *type = idata->type;
+    }
 }
 
 int
@@ -695,23 +750,32 @@ egg_toolbars_model_get_item_id (EggToolbarsModel *t,
 			        const char       *type,
 			        const char       *name)
 {
-  EggToolbarsModelClass *klass = EGG_TOOLBARS_MODEL_GET_CLASS (t);
-  return klass->get_item_id (t, type, name);
+  char *retval;
+
+  g_signal_emit (t, egg_toolbars_model_signals[GET_ITEM_ID], 0, type, name, &retval);
+
+  return retval;
 }
 
 char *
-egg_toolbars_model_get_item_name (EggToolbarsModel *t,
+egg_toolbars_model_get_item_data (EggToolbarsModel *t,
 				  const char       *type,
 			          const char       *id)
 {
-  EggToolbarsModelClass *klass = EGG_TOOLBARS_MODEL_GET_CLASS (t);
-  return klass->get_item_name (t, type, id);
+  char *retval;
+
+  g_signal_emit (t, egg_toolbars_model_signals[GET_ITEM_DATA], 0, type, id, &retval);
+
+  return retval;
 }
 
 char *
 egg_toolbars_model_get_item_type (EggToolbarsModel *t,
 				  GdkAtom type)
 {
-  EggToolbarsModelClass *klass = EGG_TOOLBARS_MODEL_GET_CLASS (t);
-  return klass->get_item_type (t, type);
+  char *retval;
+
+  g_signal_emit (t, egg_toolbars_model_signals[GET_ITEM_TYPE], 0, type, &retval);
+
+  return retval;
 }
