@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmark-action.c,v 1.31 2004/05/19 18:42:22 chpe Exp $
+ *  $Id: ephy-bookmark-action.c,v 1.40 2004/07/05 00:02:09 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,11 +27,15 @@
 #include <gtk/gtktoolitem.h>
 
 #include "ephy-bookmark-action.h"
+#include "ephy-marshal.h"
+#include "ephy-dnd.h"
+#include "ephy-bookmarksbar.h"
 #include "ephy-bookmarks.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-shell.h"
 #include "ephy-string.h"
 #include "ephy-debug.h"
+#include "ephy-gui.h"
 
 #include <string.h>
 
@@ -42,13 +46,23 @@ static void ephy_bookmark_action_class_init (EphyBookmarkActionClass *class);
 
 #define EPHY_BOOKMARK_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_BOOKMARK_ACTION, EphyBookmarkActionPrivate))
 
+static GtkTargetEntry drag_targets[] =
+{
+	{ EPHY_DND_URL_TYPE, 0,	0 }
+};
+static int n_drag_targets = G_N_ELEMENTS (drag_targets);
+
 struct EphyBookmarkActionPrivate
 {
 	int bookmark_id;
 	char *location;
 	gboolean smart_url;
 	char *icon;
-	guint cache_connection;
+	guint cache_handler;
+
+	guint motion_handler;
+	gint drag_x;
+	gint drag_y;
 };
 
 enum
@@ -63,7 +77,8 @@ enum
 
 enum
 {
-	GO_LOCATION,
+	OPEN,
+	OPEN_IN_TAB,
 	LAST_SIGNAL
 };
 
@@ -112,6 +127,7 @@ create_tool_item (GtkAction *action)
 	gtk_container_add (GTK_CONTAINER (item), hbox);
 
 	button = gtk_button_new ();
+	gtk_widget_add_events (GTK_WIDGET (button), GDK_BUTTON1_MOTION_MASK);
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (hbox), button);
@@ -122,7 +138,7 @@ create_tool_item (GtkAction *action)
 	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 	g_object_set_data (G_OBJECT (item), "entry", entry);
 
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 3);
 	gtk_widget_show (hbox);
 	gtk_container_add (GTK_CONTAINER (button), hbox);
 
@@ -144,18 +160,21 @@ ephy_bookmark_action_sync_smart_url (GtkAction *action, GParamSpec *pspec, GtkWi
 {
 	if (GTK_IS_TOOL_ITEM (proxy))
 	{
-		GtkWidget *entry;
+		GtkWidget *entry, *icon;
 		gboolean smart_url;
 
 		smart_url = EPHY_BOOKMARK_ACTION (action)->priv->smart_url;
 		entry = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "entry"));
+		icon = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "icon"));
 
 		if (smart_url)
 		{
+			gtk_widget_hide (icon);
 			gtk_widget_show (entry);
 		}
 		else
 		{
+			gtk_widget_show (icon);
 			gtk_widget_hide (entry);
 		}
 	}
@@ -168,8 +187,8 @@ favicon_cache_changed_cb (EphyFaviconCache *cache,
 {
 	if (action->priv->icon && strcmp (icon, action->priv->icon) == 0)
 	{
-		g_signal_handler_disconnect (cache, action->priv->cache_connection);
-		action->priv->cache_connection = 0;
+		g_signal_handler_disconnect (cache, action->priv->cache_handler);
+		action->priv->cache_handler = 0;
 
 		g_object_notify (G_OBJECT (action), "icon");
 	}
@@ -192,9 +211,9 @@ ephy_bookmark_action_sync_icon (GtkAction *action, GParamSpec *pspec, GtkWidget
 	{
 		pixbuf = ephy_favicon_cache_get (cache, icon_location);
 
-		if (pixbuf == NULL && bma->priv->cache_connection == 0)
+		if (pixbuf == NULL && bma->priv->cache_handler == 0)
 		{
-			bma->priv->cache_connection =
+			bma->priv->cache_handler =
 				g_signal_connect_object
 					(cache, "changed",
 					 G_CALLBACK (favicon_cache_changed_cb),
@@ -202,8 +221,6 @@ ephy_bookmark_action_sync_icon (GtkAction *action, GParamSpec *pspec, GtkWidget
 		}
 	}
 
-	if (pixbuf == NULL) return;
-
 	if (GTK_IS_TOOL_ITEM (proxy))
 	{
 		GtkImage *icon;
@@ -211,9 +228,15 @@ ephy_bookmark_action_sync_icon (GtkAction *action, GParamSpec *pspec, GtkWidget
 		icon = GTK_IMAGE (g_object_get_data (G_OBJECT (proxy), "icon"));
 		g_return_if_fail (icon != NULL);
 
+		if (pixbuf == NULL)
+		{
+			pixbuf = gtk_widget_render_icon (proxy, GTK_STOCK_NEW,
+					                 GTK_ICON_SIZE_MENU, NULL);
+		}
+
 		gtk_image_set_from_pixbuf (icon, pixbuf);
 	}
-	else if (GTK_IS_MENU_ITEM (proxy))
+	else if (GTK_IS_MENU_ITEM (proxy) && pixbuf)
 	{
 		GtkWidget *image;
 
@@ -224,7 +247,10 @@ ephy_bookmark_action_sync_icon (GtkAction *action, GParamSpec *pspec, GtkWidget
 			(GTK_IMAGE_MENU_ITEM (proxy), image);
 	}
 
-	g_object_unref (pixbuf);
+	if (pixbuf)
+	{
+		g_object_unref (pixbuf);
+	}
 }
 
 static void
@@ -273,6 +299,20 @@ ephy_bookmark_action_sync_label (GtkAction *action, GParamSpec *pspec, GtkWidget
 	g_free (title);
 }
 
+static void
+open_in_tab_activate_cb (GtkWidget *widget, EphyBookmarkAction *action)
+{
+	g_signal_emit (action, signals[OPEN_IN_TAB], 0,
+		       action->priv->location, FALSE);
+}
+
+static void
+open_in_window_activate_cb (GtkWidget *widget, EphyBookmarkAction *action)
+{
+	g_signal_emit (action, signals[OPEN_IN_TAB], 0,
+		       action->priv->location, TRUE);
+}
+
 static void
 activate_cb (GtkWidget *widget, GtkAction *action)
 {
@@ -319,21 +359,162 @@ activate_cb (GtkWidget *widget, GtkAction *action)
 		}
 	}
 
-	g_signal_emit (action, signals[GO_LOCATION], 0, location);
+	if (ephy_gui_is_middle_click ())
+	{
+		g_signal_emit (action, signals[OPEN_IN_TAB], 0, location, FALSE);
+	}
+	else
+	{
+		g_signal_emit (action, signals[OPEN], 0, location);
+	}
 
 	g_free (location);
 	g_free (text);
 }
 
+static void
+stop_drag_check (EphyBookmarkAction *action, GtkWidget *widget)
+{
+	if (action->priv->motion_handler)
+	{
+		g_signal_handler_disconnect (widget, action->priv->motion_handler);
+		action->priv->motion_handler = 0;
+	}
+}
+
+static void
+drag_data_get_cb (GtkWidget *widget, GdkDragContext *context,
+		  GtkSelectionData *selection_data, guint info,
+		  guint32 time, EphyBookmarkAction *action)
+{
+	char *address = action->priv->location;
+
+	g_return_if_fail (address != NULL);
+
+	gtk_selection_data_set (selection_data, selection_data->target, 8,
+				address, strlen (address));
+}
+
+static void
+remove_from_model (GtkWidget *widget)
+{
+	EphyBookmarks *bookmarks;
+	EggToolbarsModel *model;
+	GtkWidget *item, *toolbar;
+	int pos;
+
+	item = gtk_widget_get_ancestor (widget, GTK_TYPE_TOOL_ITEM);
+	g_return_if_fail (item != NULL);
+
+	toolbar = gtk_widget_get_ancestor (widget, GTK_TYPE_TOOLBAR);
+	g_return_if_fail (toolbar != NULL);
+
+	pos = gtk_toolbar_get_item_index (GTK_TOOLBAR (toolbar),
+				          GTK_TOOL_ITEM (item));
+
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
+	model = ephy_bookmarks_get_toolbars_model (bookmarks);
+	egg_toolbars_model_remove_item (model, 0, pos);
+}
+
+static void
+drag_data_delete_cb (GtkWidget *widget, GdkDragContext *context,
+		     EphyBookmarkAction *action)
+{
+	remove_from_model (widget);
+}
+
+static gboolean
+drag_motion_cb (GtkWidget *widget, GdkEventMotion *event, EphyBookmarkAction *action)
+{
+	if (gtk_drag_check_threshold (widget, action->priv->drag_x,
+				      action->priv->drag_y, event->x, event->y))
+	{
+		GtkTargetList *target_list;
+
+		target_list = gtk_target_list_new (drag_targets, n_drag_targets);
+
+		stop_drag_check (action, widget);
+		gtk_drag_begin (widget, target_list, GDK_ACTION_MOVE |
+				GDK_ACTION_COPY, 1, (GdkEvent*)event);
+	}
+
+	return TRUE;
+}
+
+static void
+remove_activate_cb (GtkWidget *menu, GtkWidget *proxy)
+{
+	remove_from_model (proxy);
+}
+
+static void
+show_context_menu (EphyBookmarkAction *action, GtkWidget *proxy,
+		   GtkMenuPositionFunc func)
+{
+	GtkWidget *menu, *item;
+
+	menu = gtk_menu_new ();
+
+	item = gtk_menu_item_new_with_mnemonic (_("_Open in New Tab"));
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (open_in_tab_activate_cb), action);
+
+	item = gtk_menu_item_new_with_mnemonic (_("_Open in New Window"));
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (open_in_window_activate_cb), action);
+
+	item = gtk_separator_menu_item_new ();
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_REMOVE, NULL);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (remove_activate_cb), proxy);
+
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, func, proxy, 3,
+			gtk_get_current_event_time ());
+}
+
+static void
+popup_menu_cb (GtkWidget *widget, EphyBookmarkAction *action)
+{
+	if (gtk_widget_get_ancestor (widget, EPHY_TYPE_BOOKMARKSBAR))
+        {
+                show_context_menu (action, widget,
+				   ephy_gui_menu_position_under_widget);
+        }
+}
+
 static gboolean
 button_press_cb (GtkWidget *widget,
 		 GdkEventButton *event,
-		 gpointer dummy)
+		 EphyBookmarkAction *action)
 {
-	if (event->type == GDK_BUTTON_PRESS && event->button == 2)	
+	if (event->button == 3 &&
+	    gtk_widget_get_ancestor (widget, EPHY_TYPE_BOOKMARKSBAR))	
+	{
+		show_context_menu (action, widget, NULL);
+		return TRUE;
+	}
+	else if (event->button == 2)	
 	{
 		gtk_button_pressed (GTK_BUTTON (widget));
 	}
+	else if (event->button == 1 &&
+		 gtk_widget_get_ancestor (widget, EPHY_TYPE_BOOKMARKSBAR))
+	{
+		action->priv->drag_x = event->x;
+		action->priv->drag_y = event->y;
+		action->priv->motion_handler = g_signal_connect
+			(widget, "motion_notify_event", G_CALLBACK (drag_motion_cb), action);
+	}
 
 	return FALSE;
 }
@@ -341,13 +522,15 @@ button_press_cb (GtkWidget *widget,
 static gboolean
 button_release_cb (GtkWidget *widget,
                    GdkEventButton *event,
-		   gpointer dummy)
+		   EphyBookmarkAction *action)
 {
-	if (event->type == GDK_BUTTON_RELEASE && event->button == 2)	
+	if (event->button == 2)	
 	{
 		gtk_button_released (GTK_BUTTON (widget));
-
-		return TRUE;
+	}
+	else if (event->button == 1)
+	{
+		stop_drag_check (action, widget);
 	}
 
 	return FALSE;
@@ -378,10 +561,16 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 	{
 		button = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "button"));
 		g_signal_connect (button, "clicked", G_CALLBACK (activate_cb), action);
+		g_signal_connect (button, "popup_menu",
+				  G_CALLBACK (popup_menu_cb), action);
 		g_signal_connect (button, "button-press-event",
-				  G_CALLBACK (button_press_cb), NULL);
+				  G_CALLBACK (button_press_cb), action);
 		g_signal_connect (button, "button-release-event",
-				  G_CALLBACK (button_release_cb), NULL);
+				  G_CALLBACK (button_release_cb), action);
+		g_signal_connect (button, "drag_data_get",
+				  G_CALLBACK (drag_data_get_cb), action);
+		g_signal_connect (button, "drag_data_delete",
+				  G_CALLBACK (drag_data_delete_cb), action);
 
 		entry = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "entry"));
 		g_signal_connect (entry, "activate", G_CALLBACK (activate_cb), action);
@@ -480,17 +669,29 @@ ephy_bookmark_action_class_init (EphyBookmarkActionClass *class)
 	object_class->set_property = ephy_bookmark_action_set_property;
 	object_class->get_property = ephy_bookmark_action_get_property;
 
-	signals[GO_LOCATION] =
-                g_signal_new ("go_location",
+	signals[OPEN] =
+                g_signal_new ("open",
                               G_OBJECT_CLASS_TYPE (object_class),
                               G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (EphyBookmarkActionClass, go_location),
+                              G_STRUCT_OFFSET (EphyBookmarkActionClass, open),
                               NULL, NULL,
                               g_cclosure_marshal_VOID__STRING,
                               G_TYPE_NONE,
                               1,
 			      G_TYPE_STRING);
 
+	signals[OPEN_IN_TAB] =
+                g_signal_new ("open_in_tab",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (EphyBookmarkActionClass, open_in_tab),
+                              NULL, NULL,
+                              ephy_marshal_VOID__STRING_BOOLEAN,
+                              G_TYPE_NONE,
+                              2,
+			      G_TYPE_STRING,
+			      G_TYPE_BOOLEAN);
+
 	g_object_class_install_property (object_class,
                                          PROP_BOOKMARK_ID,
                                          g_param_spec_int ("bookmark_id",
@@ -569,7 +770,7 @@ bookmarks_child_changed_cb (EphyNode *node,
 			    guint property_id,
 			    GtkAction *action)
 {
-	gulong id;
+	guint id;
 
 	id = EPHY_BOOKMARK_ACTION (action)->priv->bookmark_id;
 
@@ -612,7 +813,8 @@ ephy_bookmark_action_init (EphyBookmarkAction *action)
 
 	action->priv->location = NULL;
 	action->priv->icon = NULL;
-	action->priv->cache_connection = 0;
+	action->priv->cache_handler = 0;
+	action->priv->motion_handler = 0;
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 	node = ephy_bookmarks_get_bookmarks (bookmarks);
