@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Copyright (C) 2000, 2001, 2002, 2003 Marco Pesenti Gritti
+ *  Copyright (C) 2000, 2001, 2002, 2003, 2004 Marco Pesenti Gritti
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.185 2004/05/19 18:45:34 chpe Exp $
+ *  $Id: ephy-window.c,v 1.200 2004/07/04 11:28:47 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,6 +33,8 @@
 #include "window-commands.h"
 #include "find-dialog.h"
 #include "print-dialog.h"
+#include "ephy-embed-shell.h"
+#include "ephy-embed-single.h"
 #include "ephy-shell.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
@@ -40,7 +43,9 @@
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
 #include "ephy-statusbar.h"
+#include "egg-editable-toolbar.h"
 #include "toolbar.h"
+#include "ephy-bookmarksbar.h"
 #include "popup-commands.h"
 #include "ephy-encoding-menu.h"
 #include "ephy-tabs-menu.h"
@@ -76,6 +81,8 @@ static void ephy_window_view_toolbar_cb         (GtkAction *action,
 			                         EphyWindow *window);
 static void ephy_window_view_bookmarksbar_cb    (GtkAction *action,
 			                         EphyWindow *window);
+static void ephy_window_view_popup_windows_cb	(GtkAction *action,
+						 EphyWindow *window);
 
 static GtkActionEntry ephy_menu_entries [] = {
 
@@ -240,6 +247,11 @@ static guint ephy_menu_n_entries = G_N_ELEMENTS (ephy_menu_entries);
 
 static GtkToggleActionEntry ephy_menu_toggle_entries [] =
 {
+	/* File Menu */
+	{ "FileWorkOffline", NULL, N_("_Work Offline"), NULL,
+	  N_("Toggle network status"),
+	  G_CALLBACK (window_cmd_file_work_offline), FALSE },
+
 	/* View Menu */
 	{ "ViewToolbar", NULL, N_("_Toolbar"), "<shift><control>T",
 	  N_("Show or hide toolbar"),
@@ -253,6 +265,9 @@ static GtkToggleActionEntry ephy_menu_toggle_entries [] =
 	{ "ViewFullscreen", STOCK_FULLSCREEN, N_("_Fullscreen"), "F11",
 	  N_("Browse at full screen"),
 	  G_CALLBACK (window_cmd_view_fullscreen), FALSE },
+	{ "ViewPopupWindows", EPHY_STOCK_POPUPS, N_("Popup _Windows"), NULL,
+	  N_("Show or hide unrequested popup windows from this site"),
+	  G_CALLBACK (ephy_window_view_popup_windows_cb), FALSE },
 	{ "BrowseWithCaret", NULL, N_("Selection Caret"), "F7",
 	  "",
 	  G_CALLBACK (window_cmd_browse_with_caret), FALSE }
@@ -301,6 +316,7 @@ static guint ephy_popups_n_entries = G_N_ELEMENTS (ephy_popups_entries);
 
 #define CONF_LOCKDOWN_HIDE_MENUBAR "/apps/epiphany/lockdown/hide_menubar"
 #define CONF_DESKTOP_BG_PICTURE "/desktop/gnome/background/picture_filename"
+#define INSANE_NUMBER_OF_URLS 20
 
 #define EPHY_WINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_WINDOW, EphyWindowPrivate))
 
@@ -318,6 +334,7 @@ struct EphyWindowPrivate
 	GtkWidget *menu_dock;
 	GtkWidget *exit_fullscreen_popup;
 	Toolbar *toolbar;
+	GtkWidget *bookmarksbar;
 	GtkWidget *statusbar;
 	GtkActionGroup *action_group;
 	GtkActionGroup *popups_action_group;
@@ -329,7 +346,6 @@ struct EphyWindowPrivate
 	GtkNotebook *notebook;
 	EphyTab *active_tab;
 	EphyDialog *find_dialog;
-	EphyDialog *print_dialog;
 	gboolean closing;
 	gboolean has_size;
 	guint num_tabs;
@@ -348,6 +364,7 @@ struct EphyWindowPrivate
 	guint disable_save_to_disk_notifier_id;
 	guint disable_command_line_notifier_id;
 	guint browse_with_caret_notifier_id;
+	guint allow_popups_notifier_id;
 };
 
 enum
@@ -423,13 +440,13 @@ add_widget (GtkUIManager *merge, GtkWidget *widget, EphyWindow *window)
 static void
 update_exit_fullscreen_popup_position (EphyWindow *window)
 {
+	GtkWidget *popup = window->priv->exit_fullscreen_popup;
 	GdkRectangle screen_rect;
-	int popup_height;
+	int popup_width, popup_height;
 
-	g_return_if_fail (window->priv->exit_fullscreen_popup != NULL);
+	g_return_if_fail (popup != NULL);
 
-	gtk_window_get_size (GTK_WINDOW (window->priv->exit_fullscreen_popup),
-                             NULL, &popup_height);
+	gtk_window_get_size (GTK_WINDOW (popup), &popup_width, &popup_height);
 
 	gdk_screen_get_monitor_geometry (gdk_screen_get_default (),
                         gdk_screen_get_monitor_at_window
@@ -437,8 +454,17 @@ update_exit_fullscreen_popup_position (EphyWindow *window)
                          GTK_WIDGET (window)->window),
                          &screen_rect);
 
-	gtk_window_move (GTK_WINDOW (window->priv->exit_fullscreen_popup),
-                        screen_rect.x, screen_rect.height - popup_height);
+	if (gtk_widget_get_direction (popup) == GTK_TEXT_DIR_RTL)
+	{
+		gtk_window_move (GTK_WINDOW (popup),
+				 screen_rect.x + screen_rect.width - popup_width,
+				 screen_rect.height - popup_height);
+	}
+	else
+	{
+		gtk_window_move (GTK_WINDOW (popup),
+                	        screen_rect.x, screen_rect.height - popup_height);
+	}
 }
 
 static void
@@ -450,7 +476,12 @@ size_changed_cb (GdkScreen *screen, EphyWindow *window)
 static void
 exit_fullscreen_button_clicked_cb (GtkWidget *button, EphyWindow *window)
 {
-	gtk_window_unfullscreen (GTK_WINDOW (window));
+	GtkAction *action;
+
+	action = gtk_action_group_get_action (window->priv->action_group, "ViewFullscreen");
+	g_return_if_fail (action != NULL);
+
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
 }
 
 static void
@@ -463,17 +494,18 @@ get_chromes_visibility (EphyWindow *window, gboolean *show_menubar,
 	switch (window->priv->mode)
 	{
 		case EPHY_WINDOW_MODE_NORMAL:
-			*show_menubar = flags & EPHY_EMBED_CHROME_MENUBAR;
-			*show_statusbar = flags & EPHY_EMBED_CHROME_STATUSBAR;
-			*show_toolbar = flags & EPHY_EMBED_CHROME_TOOLBAR;
-			*show_bookmarksbar = flags & EPHY_EMBED_CHROME_BOOKMARKSBAR;
+			*show_menubar = (flags & EPHY_EMBED_CHROME_MENUBAR) != 0;
+			*show_statusbar = (flags & EPHY_EMBED_CHROME_STATUSBAR) != 0;
+			*show_toolbar = (flags & EPHY_EMBED_CHROME_TOOLBAR) != 0;
+			*show_bookmarksbar = (flags & EPHY_EMBED_CHROME_BOOKMARKSBAR) != 0;
 			break;
 		case EPHY_WINDOW_MODE_FULLSCREEN:
-			*show_toolbar = flags & EPHY_EMBED_CHROME_TOOLBAR;
+			*show_toolbar = (flags & EPHY_EMBED_CHROME_TOOLBAR) != 0;
 			*show_menubar = *show_statusbar = *show_bookmarksbar = FALSE;
 			break;
 		default:
 			*show_menubar = *show_statusbar = *show_toolbar = *show_bookmarksbar = FALSE;
+			break;
 	}
 }
 
@@ -491,27 +523,10 @@ sync_chromes_visibility (EphyWindow *window)
 		(GTK_UI_MANAGER (window->ui_merge), "/menubar");
 	g_assert (menubar != NULL);
 
-	if (show_menubar)
-	{
-		gtk_widget_show (menubar);
-	}
-	else
-	{
-		gtk_widget_hide (menubar);
-	}
-
-	toolbar_set_visibility (window->priv->toolbar, show_toolbar,
-				show_bookmarksbar);
-
-
-	if (show_statusbar)
-	{
-		gtk_widget_show (window->priv->statusbar);
-	}
-	else
-	{
-		gtk_widget_hide (window->priv->statusbar);
-	}
+	g_object_set (G_OBJECT (menubar), "visible", show_menubar, NULL);
+	g_object_set (G_OBJECT (window->priv->toolbar), "visible", show_toolbar, NULL);
+	g_object_set (G_OBJECT (window->priv->bookmarksbar), "visible", show_bookmarksbar, NULL);
+	g_object_set (G_OBJECT (window->priv->statusbar), "visible", show_statusbar, NULL);
 }
 
 static void
@@ -551,6 +566,11 @@ ephy_window_fullscreen (EphyWindow *window)
                           "size-changed", G_CALLBACK (size_changed_cb),
 			  window);
 
+	egg_editable_toolbar_set_model
+		(EGG_EDITABLE_TOOLBAR (window->priv->toolbar),
+		 EGG_TOOLBARS_MODEL (
+		 	ephy_shell_get_toolbars_model (ephy_shell, TRUE)));
+
 	sync_chromes_visibility (window);
 }
 
@@ -566,6 +586,11 @@ ephy_window_unfullscreen (EphyWindow *window)
 	gtk_widget_destroy (window->priv->exit_fullscreen_popup);
 	window->priv->exit_fullscreen_popup = NULL;
 
+	egg_editable_toolbar_set_model
+		(EGG_EDITABLE_TOOLBAR (window->priv->toolbar),
+		 EGG_TOOLBARS_MODEL (
+		 	ephy_shell_get_toolbars_model (ephy_shell, FALSE)));
+
 	sync_chromes_visibility (window);
 }
 
@@ -592,7 +617,11 @@ ephy_window_state_event_cb (GtkWidget *widget, GdkEventWindowState *event, EphyW
 		action_group = window->priv->action_group;
 
 		action = gtk_action_group_get_action (action_group, "ViewFullscreen");
+		g_signal_handlers_block_by_func
+			(action, G_CALLBACK (window_cmd_view_fullscreen), window);
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), fullscreen);
+		g_signal_handlers_unblock_by_func
+			(action, G_CALLBACK (window_cmd_view_fullscreen), window);
 
 		action = gtk_action_group_get_action (action_group, "EditToolbar");
 		g_object_set (action, "sensitive", !fullscreen, NULL);
@@ -656,6 +685,48 @@ confirm_close_with_modified_forms (EphyWindow *window)
 	return response == GTK_RESPONSE_OK;
 }
 
+static void
+menubar_deactivate_cb (GtkWidget *menubar,
+		       EphyWindow *window)
+{
+	g_signal_handlers_disconnect_by_func
+		(menubar, G_CALLBACK (menubar_deactivate_cb), window);
+
+	gtk_menu_shell_deselect (GTK_MENU_SHELL (menubar));
+
+	sync_chromes_visibility (window);
+}
+
+static gboolean
+ephy_window_key_press_event (GtkWidget *widget,
+			     GdkEventKey *event)
+{
+	EphyWindow *window = EPHY_WINDOW (widget);
+	GtkWidget *menubar;
+	guint modifiers = gtk_accelerator_get_default_mod_mask ();
+
+	/* Show and activate the menubar on F10, if it isn't visible */
+        if (event->keyval == GDK_F10 && (event->state & modifiers) == 0)
+	{
+		menubar = gtk_ui_manager_get_widget
+			(GTK_UI_MANAGER (window->ui_merge), "/menubar");
+		g_return_val_if_fail (menubar != NULL , FALSE);
+	
+		if (!GTK_WIDGET_VISIBLE (menubar))
+		{
+			g_signal_connect (menubar, "deactivate",
+					  G_CALLBACK (menubar_deactivate_cb), window);
+
+			gtk_widget_show (menubar);
+			gtk_menu_shell_select_first (GTK_MENU_SHELL (menubar), FALSE);
+
+			return TRUE;
+		}
+        }
+
+	return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
+}
+
 static gboolean
 ephy_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, EphyWindow *window)
 {
@@ -671,7 +742,7 @@ ephy_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, EphyWindow *win
 		EphyEmbed *embed;
 
 		embed = ephy_window_get_active_embed (window);
-		ephy_embed_print_preview_close (embed);
+		ephy_embed_set_print_preview_mode (embed, FALSE);
 
 		ephy_window_set_print_preview (window, FALSE);
 	}
@@ -1244,6 +1315,64 @@ sync_tab_security (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 
 }
 
+static void
+sync_tab_popup_windows (EphyTab *tab,
+			GParamSpec *pspec,
+			EphyWindow *window)
+{
+	guint num_popups = 0;
+	char *tooltip = NULL;
+
+	g_object_get (G_OBJECT (tab),
+		      "hidden-popup-count", &num_popups,
+		      NULL);
+
+	if (num_popups > 0)
+	{
+		tooltip = g_strdup_printf (ngettext ("%d hidden popup window",
+						     "%d hidden popup windows",
+						     num_popups),
+					   num_popups);
+	}
+
+	ephy_statusbar_set_popups_state
+		(EPHY_STATUSBAR (window->priv->statusbar),
+		 tooltip == NULL,
+		 tooltip);
+
+	g_free (tooltip);
+}
+
+static void
+sync_tab_popups_allowed (EphyTab *tab,
+			 GParamSpec *pspec,
+			 EphyWindow *window)
+{
+	GtkAction *action;
+	gboolean allow;
+
+	g_return_if_fail (EPHY_IS_TAB (tab));
+	g_return_if_fail (EPHY_IS_WINDOW (window));
+
+	action = gtk_action_group_get_action (window->priv->action_group,
+					      "ViewPopupWindows");
+	g_return_if_fail (GTK_IS_ACTION (action));
+
+	g_object_get (G_OBJECT (tab), "popups-allowed", &allow, NULL);
+
+	g_signal_handlers_block_by_func
+		(G_OBJECT (action),
+		 G_CALLBACK (ephy_window_view_popup_windows_cb),
+		 window);
+
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), allow);
+
+	g_signal_handlers_unblock_by_func
+		(G_OBJECT (action),
+		 G_CALLBACK (ephy_window_view_popup_windows_cb),
+		 window);
+}
+
 static void
 sync_tab_load_status (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 {
@@ -1351,6 +1480,22 @@ sync_tab_zoom (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 	g_object_set (action, "sensitive", can_zoom_normal, NULL);
 }
 
+static void
+network_status_changed (EphyEmbedSingle *single,
+			gboolean offline,
+			EphyWindow *window)
+{
+	GtkAction *action;
+
+	action = gtk_action_group_get_action (window->priv->action_group,
+					      "FileWorkOffline");
+	g_signal_handlers_block_by_func
+		(action, G_CALLBACK (window_cmd_file_work_offline), window);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), offline);
+	g_signal_handlers_unblock_by_func
+		(action, G_CALLBACK (window_cmd_file_work_offline), window);	
+}
+
 static void
 popup_menu_at_coords (GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
 		      gpointer user_data)
@@ -1380,7 +1525,6 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	gboolean framed, has_background, can_open_in_new;
 	GtkWidget *widget;
 	EphyEmbedEventType type;
-	gboolean showing_edit_actions = FALSE;
 
 	/* Do not show the menu in print preview mode */
 	if (window->priv->mode == EPHY_WINDOW_MODE_PRINT_PREVIEW)
@@ -1405,8 +1549,8 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	}
 	else if (context & EMBED_CONTEXT_LINK)
 	{
-		showing_edit_actions = TRUE;
 		popup = "/EphyLinkPopup";
+		update_edit_actions_sensitivity (window, TRUE);
 	}
 	else if (context & EMBED_CONTEXT_IMAGE)
 	{
@@ -1414,14 +1558,14 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	}
 	else if (context & EMBED_CONTEXT_INPUT)
 	{
-		showing_edit_actions = TRUE;
 		popup = "/EphyInputPopup";
+		update_edit_actions_sensitivity (window, FALSE);
 	}
 	else
 	{
-		showing_edit_actions = TRUE;
 		popup = framed ? "/EphyFramedDocumentPopup" :
 				 "/EphyDocumentPopup";
+		update_edit_actions_sensitivity (window, TRUE);
 	}
 
 	widget = gtk_ui_manager_get_widget (GTK_UI_MANAGER (window->ui_merge),
@@ -1437,11 +1581,6 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	action = gtk_action_group_get_action (action_group, "OpenLinkInNewTab");
 	g_object_set (action, "sensitive", can_open_in_new, FALSE);
 
-	if (showing_edit_actions)
-	{
-		update_edit_actions_sensitivity (window, TRUE);
-	}
-
 	g_object_set_data_full (G_OBJECT (window), "context_event",
 				g_object_ref (event),
 				(GDestroyNotify)g_object_unref);
@@ -1521,6 +1660,12 @@ ephy_window_set_active_tab (EphyWindow *window, EphyTab *new_tab)
 		g_signal_handlers_disconnect_by_func (G_OBJECT (old_tab),
 						      G_CALLBACK (sync_tab_security),
 						      window);
+		g_signal_handlers_disconnect_by_func (G_OBJECT (old_tab),
+						      G_CALLBACK (sync_tab_popup_windows),
+						      window);
+		g_signal_handlers_disconnect_by_func (G_OBJECT (old_tab),
+						      G_CALLBACK (sync_tab_popups_allowed),
+						      window);
 		g_signal_handlers_disconnect_by_func (G_OBJECT (old_tab),
 						      G_CALLBACK (sync_tab_title),
 						      window);
@@ -1548,6 +1693,8 @@ ephy_window_set_active_tab (EphyWindow *window, EphyTab *new_tab)
 		sync_tab_message	(new_tab, NULL, window);
 		sync_tab_navigation	(new_tab, NULL, window);
 		sync_tab_security	(new_tab, NULL, window);
+		sync_tab_popup_windows	(new_tab, NULL, window);
+		sync_tab_popups_allowed	(new_tab, NULL, window);
 		sync_tab_title		(new_tab, NULL, window);
 		sync_tab_zoom		(new_tab, NULL, window);
 
@@ -1579,6 +1726,14 @@ ephy_window_set_active_tab (EphyWindow *window, EphyTab *new_tab)
 					 "notify::security-level",
 					 G_CALLBACK (sync_tab_security),
 					 window, 0);
+		g_signal_connect_object (G_OBJECT (new_tab),
+					 "notify::hidden-popup-count",
+					 G_CALLBACK (sync_tab_popup_windows),
+					 window, 0);
+		g_signal_connect_object (G_OBJECT (new_tab),
+					 "notify::popups-allowed",
+					 G_CALLBACK (sync_tab_popups_allowed),
+					 window, 0);
 		g_signal_connect_object (G_OBJECT (new_tab),
 					 "notify::title",
 					 G_CALLBACK (sync_tab_title),
@@ -1796,6 +1951,9 @@ ephy_window_get_property (GObject *object,
 		case PROP_ACTIVE_TAB:
 			g_value_set_object (value, window->priv->active_tab);
 			break;
+		case PROP_CHROME:
+			g_value_set_flags (value, window->priv->chrome);
+			break;
 	}
 }
 
@@ -1815,6 +1973,7 @@ ephy_window_class_init (EphyWindowClass *klass)
 	gtkobject_class->destroy = ephy_window_destroy;
 
 	widget_class->show = ephy_window_show;
+	widget_class->key_press_event = ephy_window_key_press_event;
 
 	g_object_class_install_property (object_class,
 					 PROP_ACTIVE_TAB,
@@ -1913,10 +2072,41 @@ browse_with_caret_notifier (GConfClient *client,
 				      eel_gconf_get_boolean (CONF_BROWSE_WITH_CARET));
 }
 
+static void
+allow_popups_notifier (GConfClient *client,
+		       guint cnxn_id,
+		       GConfEntry *entry,
+		       EphyWindow *window)
+{
+	GList *tabs;
+	EphyTab *tab;
+
+	g_return_if_fail (EPHY_IS_WINDOW (window));
+
+	tabs = ephy_window_get_tabs (window);
+
+	for (; tabs; tabs = g_list_next (tabs))
+	{
+		tab = EPHY_TAB (tabs->data);
+		g_return_if_fail (EPHY_IS_TAB (tab));
+
+		g_object_notify (G_OBJECT (tab), "popups-allowed");
+	}
+}
+
+static void
+action_request_forward_cb (GObject *toolbar,
+			   const char *name,
+                	   GObject *bookmarksbar)
+{
+	g_signal_emit_by_name (bookmarksbar, "action_request", name);
+}
+
 static void
 ephy_window_init (EphyWindow *window)
 {
 	EphyExtension *manager;
+	EphyEmbedSingle *single;
 
 	LOG ("EphyWindow initialising %p", window)
 
@@ -1928,6 +2118,7 @@ ephy_window_init (EphyWindow *window)
 	window->priv->exit_fullscreen_popup = NULL;
 	window->priv->num_tabs = 0;
 	window->priv->has_size = FALSE;
+	window->priv->chrome = EPHY_EMBED_CHROME_ALL;
 	window->priv->should_save_chrome = FALSE;
 	window->priv->mode = EPHY_WINDOW_MODE_NORMAL;
 
@@ -1942,6 +2133,7 @@ ephy_window_init (EphyWindow *window)
 	gtk_box_pack_start (GTK_BOX (window->priv->main_vbox),
 			    GTK_WIDGET (window->priv->notebook),
 			    TRUE, TRUE, 0);
+	gtk_widget_show (GTK_WIDGET (window->priv->notebook));
 
 	window->priv->statusbar = ephy_statusbar_new ();
 	gtk_box_pack_start (GTK_BOX (window->priv->main_vbox),
@@ -1958,8 +2150,21 @@ ephy_window_init (EphyWindow *window)
 	window->priv->enc_menu = ephy_encoding_menu_new (window);
 	window->priv->bmk_menu = ephy_bookmarks_menu_new (window);
 
-	/* create the toolbar */
+	/* create the toolbars */
 	window->priv->toolbar = toolbar_new (window);
+	window->priv->bookmarksbar = ephy_bookmarksbar_new (window);
+
+	/* forward the toolbar's action_request signal to the bookmarks toolbar,
+	 * so the user can also have bookmarks on the normal toolbar
+	 */
+	g_signal_connect (window->priv->toolbar, "action_request",
+			  G_CALLBACK (action_request_forward_cb),
+			  window->priv->bookmarksbar);
+
+	/* Add the toolbars to the window */
+	gtk_box_pack_end (GTK_BOX (window->priv->menu_dock),
+			  window->priv->bookmarksbar,
+			  FALSE, FALSE, 0);
 	gtk_box_pack_end (GTK_BOX (window->priv->menu_dock),
 			  GTK_WIDGET (window->priv->toolbar),
 			  FALSE, FALSE, 0);
@@ -1970,9 +2175,14 @@ ephy_window_init (EphyWindow *window)
 	manager = EPHY_EXTENSION (ephy_shell_get_extensions_manager (ephy_shell));
 	ephy_extension_attach_window (manager, window);
 
-	/* show widgets */
-	gtk_widget_show (GTK_WIDGET (window->priv->toolbar));
-	gtk_widget_show (GTK_WIDGET (window->priv->notebook));
+	/* We only set the model now after attaching the extensions, so that
+	 * extensions already have created their actions which may be on
+	 * the toolbar
+	 */
+	egg_editable_toolbar_set_model
+		(EGG_EDITABLE_TOOLBAR (window->priv->toolbar),
+		 EGG_TOOLBARS_MODEL
+			 (ephy_shell_get_toolbars_model (ephy_shell, FALSE)));
 
 	g_signal_connect (window, "window-state-event",
 			  G_CALLBACK (ephy_window_state_event_cb),
@@ -2020,6 +2230,18 @@ ephy_window_init (EphyWindow *window)
 		(CONF_BROWSE_WITH_CARET,
 		 (GConfClientNotifyFunc)browse_with_caret_notifier, window);
 
+	window->priv->allow_popups_notifier_id = eel_gconf_notification_add
+		(CONF_SECURITY_ALLOW_POPUPS,
+		 (GConfClientNotifyFunc)allow_popups_notifier, window);
+
+	/* network status */
+	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (embed_shell));
+	network_status_changed (single,
+			        ephy_embed_single_get_offline_mode (single),
+				window);
+	g_signal_connect (single, "network-status",
+			  G_CALLBACK (network_status_changed), window);
+
 	/* ensure the UI is updated */
 	gtk_ui_manager_ensure_update (GTK_UI_MANAGER (window->ui_merge));
 
@@ -2030,6 +2252,11 @@ static void
 ephy_window_finalize (GObject *object)
 {
         EphyWindow *window = EPHY_WINDOW (object);
+	GObject *single;
+
+	single = ephy_embed_shell_get_embed_single (embed_shell);
+	g_signal_handlers_disconnect_by_func
+		(single, G_CALLBACK (network_status_changed), window);
 
 	eel_gconf_notification_remove (window->priv->disable_arbitrary_url_notifier_id);
 	eel_gconf_notification_remove (window->priv->disable_bookmark_editing_notifier_id);
@@ -2040,17 +2267,13 @@ ephy_window_finalize (GObject *object)
 	eel_gconf_notification_remove (window->priv->disable_save_to_disk_notifier_id);
 	eel_gconf_notification_remove (window->priv->disable_command_line_notifier_id);
 	eel_gconf_notification_remove (window->priv->browse_with_caret_notifier_id);
+	eel_gconf_notification_remove (window->priv->allow_popups_notifier_id);
 
 	if (window->priv->find_dialog)
 	{
 		g_object_unref (G_OBJECT (window->priv->find_dialog));
 	}
 
-	if (window->priv->print_dialog)
-	{
-		g_object_unref (G_OBJECT (window->priv->print_dialog));
-	}
-
 	g_object_unref (window->priv->fav_menu);
 	g_object_unref (window->priv->enc_menu);
 	g_object_unref (window->priv->tabs_menu);
@@ -2147,9 +2370,7 @@ ephy_window_set_print_preview (EphyWindow *window, gboolean enabled)
  * ephy_window_get_toolbar:
  * @window: an #EphyWindow
  *
- * Returns this window's toolbar as an #EggEditableToolbar. Note that this
- * toolbar is uneditable outside of Epiphany itself (i.e., extensions should not
- * access it).
+ * Returns this window's toolbar as an #EggEditableToolbar.
  *
  * Return value: an #EggEditableToolbar
  **/
@@ -2161,6 +2382,22 @@ ephy_window_get_toolbar (EphyWindow *window)
 	return GTK_WIDGET (window->priv->toolbar);
 }
 
+/**
+ * ephy_window_get_bookmarksbar:
+ * @window: an #EphyWindow
+ *
+ * Returns this window's bookmarks toolbar, which is an #EggEditableToolbar.
+ *
+ * Return value: an #EggEditableToolbar
+ **/
+GtkWidget *
+ephy_window_get_bookmarksbar (EphyWindow *window)
+{
+	g_return_val_if_fail (EPHY_IS_WINDOW (window), NULL);
+
+	return GTK_WIDGET (window->priv->bookmarksbar);
+}
+
 /**
  * ephy_window_get_notebook:
  * @window: an #EphyWindow
@@ -2215,8 +2452,8 @@ ephy_window_add_tab (EphyWindow *window,
 
 	widget = GTK_WIDGET(ephy_tab_get_embed (tab));
 
-	ephy_notebook_insert_tab (EPHY_NOTEBOOK (window->priv->notebook),
-				  tab, position, jump_to);
+	ephy_notebook_add_tab (EPHY_NOTEBOOK (window->priv->notebook),
+			       tab, position, jump_to);
 }
 
 /**
@@ -2297,7 +2534,7 @@ ephy_window_remove_tab (EphyWindow *window,
  **/
 void
 ephy_window_load_url (EphyWindow *window,
-			const char *url)
+		      const char *url)
 {
 	EphyEmbed *embed;
 
@@ -2404,18 +2641,9 @@ ephy_window_get_active_embed (EphyWindow *window)
 GList *
 ephy_window_get_tabs (EphyWindow *window)
 {
-	GtkNotebook *notebook;
-	GList *list = NULL;
-	int i, num;
+	g_return_val_if_fail (EPHY_IS_WINDOW (window), NULL);
 
-	notebook = GTK_NOTEBOOK (window->priv->notebook);
-	num = gtk_notebook_get_n_pages (notebook);
-	for (i = 0; i < num; i++)
-	{
-		list = g_list_prepend (list, gtk_notebook_get_nth_page (notebook, i));
-	}
-
-	return g_list_reverse (list);
+	return gtk_container_get_children (GTK_CONTAINER (window->priv->notebook));
 }
 
 static void
@@ -2424,7 +2652,6 @@ update_embed_dialogs (EphyWindow *window,
 {
 	EphyEmbed *embed;
 	EphyDialog *find_dialog = window->priv->find_dialog;
-	EphyDialog *print_dialog = window->priv->print_dialog;
 
 	embed = ephy_tab_get_embed (tab);
 
@@ -2434,13 +2661,6 @@ update_embed_dialogs (EphyWindow *window,
 			(EPHY_EMBED_DIALOG(find_dialog),
 			 embed);
 	}
-
-	if (print_dialog)
-	{
-		ephy_embed_dialog_set_embed
-			(EPHY_EMBED_DIALOG(print_dialog),
-			 embed);
-	}
 }
 
 static void
@@ -2494,54 +2714,6 @@ ephy_window_find (EphyWindow *window)
 	ephy_dialog_show (window->priv->find_dialog);
 }
 
-/**
- * ephy_window_print:
- * @window: an #EphyWindow
- *
- * Displays @window's Print dialog.
- **/
-void
-ephy_window_print (EphyWindow *window)
-{
-	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINTING))
-	{
-		return;
-	}
-
-	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINT_SETUP) ||
-	    eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_COMMAND_LINE))
-	{
-		EphyEmbed *embed;
-		EmbedPrintInfo *info;
-
-		info = ephy_print_get_print_info ();
-
-		embed = ephy_window_get_active_embed (window);
-		g_return_if_fail (EPHY_IS_EMBED (embed));
-
-		ephy_embed_print (embed, info);
-
-		ephy_print_info_free (info);
-
-		return;
-	}
-
-	if (window->priv->print_dialog == NULL)
-	{
-		EphyDialog *dialog;
-		EphyEmbed *embed;
-
-		embed = ephy_window_get_active_embed (window);
-		dialog = ephy_print_dialog_new  (GTK_WIDGET (window), embed, FALSE);
-
-		window->priv->print_dialog = dialog;
-		g_object_add_weak_pointer(G_OBJECT (dialog),
-					  (gpointer *) &window->priv->print_dialog);
-	}
-
-	ephy_dialog_show (window->priv->print_dialog);
-}
-
 /**
  * ephy_window_set_zoom:
  * @window: an #EphyWindow
@@ -2575,7 +2747,54 @@ ephy_window_set_zoom (EphyWindow *window,
 
 	if (zoom != current_zoom)
 	{
-		ephy_embed_set_zoom (embed, zoom, TRUE);
+		ephy_embed_set_zoom (embed, zoom);
+	}
+}
+
+void
+ephy_window_load_in_tabs (EphyWindow *window, EphyTab *tab, GList *uri_list)
+{
+	EphyEmbed *embed = NULL;
+	GList *l;
+	guint num = 0;
+	GnomeVFSURI *uri;
+
+	if (tab != NULL)
+	{
+		embed = ephy_tab_get_embed (tab);
+		g_return_if_fail (EPHY_IS_EMBED (embed));
+	}
+
+	l = uri_list;
+	while (l != NULL && num < INSANE_NUMBER_OF_URLS)
+	{
+		gchar *url = NULL;
+
+		uri = (GnomeVFSURI*) l->data;
+		url = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+
+		if (num == 0 && embed != NULL)
+		{
+			/**
+			 * The first url is special: if the drag was to an
+			 * existing tab, load it there
+			 */
+			ephy_embed_load_url (embed, url);
+		}
+		else
+		{
+			tab = ephy_shell_new_tab (ephy_shell, window,
+						  tab, url,
+						  EPHY_NEW_TAB_OPEN_PAGE |
+						  EPHY_NEW_TAB_IN_EXISTING_WINDOW |
+						  (tab ? EPHY_NEW_TAB_APPEND_AFTER :
+							 EPHY_NEW_TAB_APPEND_LAST));
+		}
+
+		g_free (url);
+		url = NULL;
+		l = l->next;
+		++num;
 	}
 }
 
@@ -2632,3 +2851,27 @@ ephy_window_view_bookmarksbar_cb (GtkAction *action,
 	sync_chrome_with_view_toggle (action, window,
 				      EPHY_EMBED_CHROME_BOOKMARKSBAR);
 }
+
+static void
+ephy_window_view_popup_windows_cb (GtkAction *action,
+				   EphyWindow *window)
+{
+	EphyTab *tab;
+	gboolean allow;
+
+	g_return_if_fail (EPHY_IS_WINDOW (window));
+
+	tab = ephy_window_get_active_tab (window);
+	g_return_if_fail (EPHY_IS_TAB (tab));
+
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
+	{
+		allow = TRUE;
+	}
+	else
+	{
+		allow = FALSE;
+	}
+
+	g_object_set (G_OBJECT (tab), "popups-allowed", allow, NULL);
+}
