@@ -15,13 +15,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.h,v 1.23.4.1 2004/03/22 09:21:31 marco Exp $
+ *  $Id: ephy-window.h,v 1.28 2004/06/23 18:28:31 marco Exp $
  */
 
 #ifndef EPHY_WINDOW_H
 #define EPHY_WINDOW_H
 
 #include "ephy-embed.h"
+#include "ephy-tab.h"
 #include "ephy-dialog.h"
 
 #include <glib-object.h>
@@ -38,16 +39,15 @@ G_BEGIN_DECLS
 #define EPHY_WINDOW_GET_CLASS(o)(G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_WINDOW, EphyWindowClass))
 
 typedef struct EphyWindowClass EphyWindowClass;
-typedef struct EphyWindow EphyWindow;
+typedef struct _EphyWindow EphyWindow;
 typedef struct EphyWindowPrivate EphyWindowPrivate;
 
-struct EphyWindow
+struct _EphyWindow
 {
         GtkWindow parent;
 	/*< private >*/
         EphyWindowPrivate *priv;
 
-	/* Public to toolbar and statusbar, dont use outside */
 	/*< public >*/
 	GObject *ui_merge;
 };
@@ -58,7 +58,6 @@ struct EphyWindowClass
 };
 
 /* Include the header down here to resolve circular dependency */
-#include "ephy-tab.h"
 #include "ephy-notebook.h"
 
 GType		  ephy_window_get_type		  (void);
@@ -72,6 +71,8 @@ void		  ephy_window_set_print_preview	  (EphyWindow *window,
 
 GtkWidget	 *ephy_window_get_toolbar	  (EphyWindow *window);
 
+GtkWidget	 *ephy_window_get_bookmarksbar	  (EphyWindow *window);
+
 GtkWidget	 *ephy_window_get_notebook	  (EphyWindow *window);
 
 GtkWidget	 *ephy_window_get_statusbar	  (EphyWindow *window);
@@ -90,6 +91,10 @@ void		  ephy_window_jump_to_tab	  (EphyWindow *window,
 void		  ephy_window_load_url		  (EphyWindow *window,
 						   const char *url);
 
+void		  ephy_window_load_in_tabs	  (EphyWindow *window,
+						   EphyTab *first_tab,
+						   GList *uri_list);
+
 void		  ephy_window_set_zoom		  (EphyWindow *window,
 						   float zoom);
 
